#include <userint.h>
#include <Ivi.h>

/*****************************************************************************/
/*   Copyright 1998 National Instruments Corporation.  All Rights Reserved.  */
/*****************************************************************************/

/*=============== Q8384 - Optical Spectrum Analyzer =========================*/
/* LabWindows/CVI 5.0 Instrument Driver										 */
/* Original Release: May/29/2000											 */
/* By: Ai Onishi,    Supervisor: Yohei Hirakoso								 */
/*                   Software R&D Section 1, ADVANTEST CORPORATION			 */
/*                   Home Page; http://www.advantest.co.jp					 */
/*                   e-mail;    info@advantest.co.jp						 */
/*---------------------------------------------------------------------------*/
/* Modification History:													 */
/*		   Version 1.0 :  Jun/30/2000										 */
/*         Version 2.0 :  May/30/2001										 */
/*		   Version 2.1 :  Sep/5/2001										 */
/*===========================================================================*/

#include <visa.h>
#include <ansi_c.h>
#include <formatio.h>
#include "avq8384.h"

#define avq8384_REVISION    "Rev 2.1, 09/2001, CVI 5.0" /*  Instrument driver revision */
#define BUFFER_SIZE         1024L         				/* File I/O buffer size */

#define CNT_F_MIN		176349.0
#define CNT_F_MAX		499654.0
#define SPN_F_MIN		0.0
#define SPN_F_MAX		323306.0

#define CNT_W_MIN		600.0
#define CNT_W_MAX		1700.0
#define SPN_W_MIN		0.0
#define SPN_W_MAX		1100.0

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/

ViString spanModeArr[] = {"SPA","STA","STO","LSP","FSP"};
ViString levModeArr[] = {"PKL", "CUL", "MNH", "MXH","RAU"};
ViString LvlUntArr[] = {"DBM","MW","UW","NW"};
ViString unitUmNmArr[] = {"UM","NM"};
ViString unitSecMsecArr[] ={"SEC", "MSEC"};
ViString unitLvlPerArr[] = {"DBM","DB","MW","UW", "NW","PC"};
ViString measDataActArr[] = {"SAV","RCL","DMD"};

/* Add  (Ver.2.0) */
ViString xAxisUnitArr[] = {"NM","GHZ"};
ViString avgModeArr[] = {"AVG","AVS","SMN"};
ViString unitGhzThzArr[] = {"GHZ","THZ"};
ViString mltTrCtlArr[] = {"MNT","MPT","MCU","MDC","MDA"};

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE  ============================*/
/*****************************************************************************/
/* avq8384_stringValPair is used in the avq8384_errorMessage function        */
/* avq8384_statusDataRanges is used to track session dependent status & ranges*/
/*===========================================================================*/
typedef struct  avq8384_stringValPair
{
   ViStatus stringVal;
   ViString stringName;
}  avq8384_tStringValPair;
 
struct avq8384_statusDataRanges {
    ViInt16 traceRange;
    ViChar instrDriverRevision[256];
};

typedef struct avq8384_statusDataRanges *avq8384_instrRange;

/*****************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) =================*/
/*****************************************************************************/
ViBoolean avq8384_invalidViBooleanRange (ViBoolean val);
ViBoolean avq8384_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max);
ViBoolean avq8384_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean avq8384_invalidViUInt8Range (ViUInt8 val, ViUInt8 min, ViUInt8 max);
ViBoolean avq8384_invalidViUInt16Range (ViUInt16 val, ViUInt16 min, ViUInt16 max);
ViBoolean avq8384_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max);
ViBoolean avq8384_invalidViReal32Range (ViReal32 val, ViReal32 min, ViReal32 max);
ViBoolean avq8384_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max);
ViStatus avq8384_waitOnVisaEvent (ViSession instrSession, ViEventType eventTypeIn,
	ViUInt32 tmoIn, ViPUInt16 STB);
ViStatus avq8384_initCleanUp (ViSession openRMSession, ViPSession openInstrSession,
	ViStatus currentStatus);
ViStatus avq8384_defaultInstrSetup (ViSession openInstrSession);

/*****************************************************************************/
/*====== USER-CALLABLE FUNCTIONS (Exportable Functions) =====================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function opens the instrument, queries the instrument      */
/*           for its ID, and initializes the instrument to a known state.    */
/*===========================================================================*/

ViStatus CVIFUNC avq8384_Initialize (ViRsrc resourceName, ViBoolean IDQuery,
                                     ViBoolean resetDevice,
                                     ViSession *instrumentHandle)
{
    ViStatus avq8384_status = VI_SUCCESS;
    ViSession rmSession = 0;
    ViUInt32 retCnt = 0;
    ViByte rdBuffer[BUFFER_SIZE];

    /*- Check input parameter ranges ----------------------------------------*/
    if (avq8384_invalidViBooleanRange (IDQuery))
        return VI_ERROR_PARAMETER2;
    if (avq8384_invalidViBooleanRange (resetDevice))
        return VI_ERROR_PARAMETER3;

    /*- Open instrument session ---------------------------------------------*/
    if ((avq8384_status = viOpenDefaultRM (&rmSession)) < 0)
        return avq8384_status;

    if ((avq8384_status = viOpen (rmSession, resourceName, VI_NULL, VI_NULL, instrumentHandle)) < 0) {
        viClose (rmSession);
        return avq8384_status;
    }

    /*- Configure VISA Formatted I/O ----------------------------------------*/
    if ((avq8384_status = viSetAttribute (*instrumentHandle, VI_ATTR_TMO_VALUE, 10000)) < 0)
            return avq8384_initCleanUp (rmSession, instrumentHandle, avq8384_status);
  
    if ((avq8384_status = viSetBuf (*instrumentHandle, VI_READ_BUF|VI_WRITE_BUF, 40008)) < 0)
            return avq8384_initCleanUp (rmSession, instrumentHandle, avq8384_status);
  
    if ((avq8384_status = viSetAttribute (*instrumentHandle, VI_ATTR_WR_BUF_OPER_MODE,
                            VI_FLUSH_ON_ACCESS)) < 0)
            return avq8384_initCleanUp (rmSession, instrumentHandle, avq8384_status);
    
    if ((avq8384_status = viSetAttribute (*instrumentHandle, VI_ATTR_RD_BUF_OPER_MODE,
                            VI_FLUSH_ON_ACCESS)) < 0)
            return avq8384_initCleanUp (rmSession, instrumentHandle, avq8384_status);

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery) {
        if ((avq8384_status = viWrite (*instrumentHandle, "*IDN?", 5, &retCnt)) < 0)
            return avq8384_initCleanUp (rmSession, instrumentHandle, avq8384_status);
        if ((avq8384_status = viRead (*instrumentHandle, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
            return avq8384_status;

        Scan (rdBuffer, "ADVANTEST,Q8384");
        if (NumFmtdBytes () != 15) 
        {
        	Scan (rdBuffer, "ADVANTEST,Q8384");
        	if (NumFmtdBytes () != 15)       
            	return avq8384_initCleanUp (rmSession, instrumentHandle, VI_ERROR_FAIL_ID_QUERY);
        }
    }
        
    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice) {
        if ((avq8384_status = avq8384_reset (*instrumentHandle, 0)) < 0)
            return avq8384_initCleanUp (rmSession, instrumentHandle, avq8384_status);
    }       
    else  /*- Send Default Instrument Setup ---------------------------------*/
        if ((avq8384_status = avq8384_defaultInstrSetup (*instrumentHandle)) < 0)
            return avq8384_initCleanUp (rmSession, instrumentHandle, avq8384_status);
          
    return avq8384_status;
}

/*===========================================================================*/
/* Function: Center Wavelength Menu Set                                      */
/* Purpose:  This function sets the Center Wavelength parameters.			 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_CntrWvlng_MenuSet (ViSession instrumentHandle,
                                            ViInt16 centerMode,
                                            ViBoolean xaxisMode,
                                            ViReal64 centerValue,
                                            ViBoolean autoPeak)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

	switch (centerMode){
		case 0:
			if (avq8384_invalidViBooleanRange (xaxisMode))
        		return VI_ERROR_PARAMETER3;

			if (xaxisMode == 1)
			{
				if (avq8384_invalidViReal64Range (centerValue, CNT_F_MIN, CNT_F_MAX))
        			return VI_ERROR_PARAMETER4;
        		Fmt (wrtBuf, "%s<FRQ%d; CEN %fGHZ",xaxisMode ,centerValue);
			}
			else
			{
				if (avq8384_invalidViReal64Range (centerValue, CNT_W_MIN, CNT_W_MAX))
        			return VI_ERROR_PARAMETER4;
        		Fmt (wrtBuf, "%s<FRQ%d; CEN %fNM", xaxisMode, centerValue);
			}
			
			break;
			
		case 1:
			Fmt (wrtBuf, "%s<PKC");
			break;
			
		case 2:
			Fmt (wrtBuf, "%s<CUC"); 
			break;
			
		case 3:
			if (avq8384_invalidViBooleanRange (autoPeak))
        		return VI_ERROR_PARAMETER5;

			Fmt (wrtBuf, "%s<APC %d",autoPeak);
			break;
	}
	
	total += NumFmtdBytes();
	

    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avq8384_status;

    return avq8384_status;    
}


/*===========================================================================*/
/* Function: Span Wavelength Setup											 */
/*  This function sets Reference level Mode & Value							 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_WvLngSpn_MenuSet (ViSession instrumentHandle,
                                           ViInt16 spanMode, ViBoolean xaxisMode,
                                           ViReal64 value)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

    if (avq8384_invalidViInt16Range (spanMode, 0, 4))
        return VI_ERROR_PARAMETER2;

	if (spanMode > 2){ 									/* Cursor Span, FULL */
		Fmt (wrtBuf, "%s<%s",spanModeArr[spanMode]);     
	}
	else												/* Else mode (with value) */
	{
		if (avq8384_invalidViBooleanRange (xaxisMode))
        	return VI_ERROR_PARAMETER3;
        
        if (xaxisMode)
        {
        	if (avq8384_invalidViReal64Range (value, 0.00, 323306))
        		return VI_ERROR_PARAMETER4;
        }
        else
        {
			if (avq8384_invalidViReal64Range (value, 0.00, 1100.00))
        		return VI_ERROR_PARAMETER4;
        }
		Fmt (wrtBuf, "%s<%s %f%s",spanModeArr[spanMode],value,xAxisUnitArr[xaxisMode]);  
	}

    total += NumFmtdBytes();

  	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avq8384_status;

    return avq8384_status;        
}

/*===========================================================================*/
/* Function: Center Span
   This function sets Center Wavelength & Wavelength Span.					 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_XaxisMode_Set (ViSession instrumentHandle,
                                        ViBoolean xaxisMode)
{
   ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

   if (avq8384_invalidViBooleanRange (xaxisMode))
      return VI_ERROR_PARAMETER2;
   Fmt (wrtBuf, "%s<FRQ%d", xaxisMode);
   
   total += NumFmtdBytes();

    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avq8384_status;

    return avq8384_status;   
}

/*===========================================================================*/
/* Function: Center Span
   This function sets Center Wavelength & Wavelength Span.					 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_CenterSpan_Set (ViSession instrumentHandle,
                                         ViBoolean xaxisMode, 
                                         ViReal64 center,
                                         ViReal64 span)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

	if (avq8384_invalidViBooleanRange (xaxisMode))
        return VI_ERROR_PARAMETER2;
        
    if (xaxisMode)
	{
		if (avq8384_invalidViReal64Range (center, CNT_F_MIN, CNT_F_MAX))
        	return VI_ERROR_PARAMETER3;
		if (avq8384_invalidViReal64Range (span, SPN_F_MIN, SPN_F_MAX))
        	return VI_ERROR_PARAMETER4;	
        Fmt (wrtBuf, "%s<FRQ%d; CEN %fGHZ; SPA %fGHZ",xaxisMode, center, span);      	
	}
	else
	{
		if (avq8384_invalidViReal64Range (center, CNT_W_MIN, CNT_W_MAX))
        	return VI_ERROR_PARAMETER3;
		if (avq8384_invalidViReal64Range (span, SPN_W_MIN, SPN_W_MAX))
        	return VI_ERROR_PARAMETER4;
        Fmt (wrtBuf, "%s<FRQ%d; CEN %fNM; SPA %fNM", xaxisMode, center, span);      	
	 }
	 
    total += NumFmtdBytes();

    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avq8384_status;

    return avq8384_status;    
}


/*===========================================================================*/
/* Function: Start Stop
   This function sets Start & Stop Wavelength.								 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_StartStop_Set (ViSession instrumentHandle,
                                        ViBoolean xaxisMode,
                                        ViReal64 start,
                                        ViReal64 stop)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;
	if (avq8384_invalidViBooleanRange (xaxisMode))
        return VI_ERROR_PARAMETER2;
        

	Fmt (wrtBuf, "%s<FRQ%d; STA %f%s; STO %f%s", xaxisMode, start, xAxisUnitArr[xaxisMode], stop, xAxisUnitArr[xaxisMode]);
	
    total += NumFmtdBytes();

    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avq8384_status;

    return avq8384_status;    	    
}

/*===========================================================================*/
/* Function: Wavelength Set													 */
/*   This function sets Center/Span/Start/Stop.								 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_Wavelength_Set (ViSession instrumentHandle,
                                         ViInt16 wavelengthMode,
                                         ViReal64 value,
                                         ViBoolean XAxisUnit)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

	if (avq8384_invalidViBooleanRange (XAxisUnit))
   		return VI_ERROR_PARAMETER4;   
	
	if (avq8384_invalidViInt16Range (wavelengthMode, 0, 3))
        return VI_ERROR_PARAMETER2;

/* No Range Check */
/*    
	switch (wavelengthMode)
	{
		case 0:
			if (XAxisMode)
			{
				if (avq8384_invalidViReal64Range (value, CNT_F_MIN, CNT_F_MAX))
        			return VI_ERROR_PARAMETER3;
			}
			else
			{
				if (avq8384_invalidViReal64Range (value, CNT_W_MIN, CNT_W_MAX))
        			return VI_ERROR_PARAMETER3;
			}
			Fmt (wrtBuf, "%s<CEN");
			break;
		case 1:
			if (avq8384_invalidViReal64Range (value, SPN_W_MIN, SPN_W_MAX))
        		return VI_ERROR_PARAMETER4;
			Fmt (wrtBuf, "%s<SPA");    
			break;
		case 2:
			Fmt (wrtBuf, "%s<STA");    
			break;
		case 3:
			Fmt (wrtBuf, "%s<STO");    
			break;
	}
*/

	switch (wavelengthMode)
	{
		case 0:
			Fmt (wrtBuf, "%s<CEN");
			break;
		case 1:
			Fmt (wrtBuf, "%s<SPA");    
			break;
		case 2:
			Fmt (wrtBuf, "%s<STA");    
			break;
		case 3:
			Fmt (wrtBuf, "%s<STO");    
			break;
	}

    total += NumFmtdBytes();
	Fmt (wrtBuf, "%s[a]< %f%s", value, xAxisUnitArr[XAxisUnit]);
    total += NumFmtdBytes();

    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avq8384_status;

    return avq8384_status;    
}

/*===========================================================================*/
/* Function: Reference Level Setup											 */
/* This function sets Reference level Mode & Value							 */
/*===========================================================================*/

ViStatus CVIFUNC avq8384_RefLvl_MenuSet (ViSession instrumentHandle,
                                         ViInt16 levelMode, ViReal64 refLevel,
                                         ViInt16 levelUnit, ViBoolean modeSwitch)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

    if (avq8384_invalidViInt16Range (levelMode, 0, 5))
        return VI_ERROR_PARAMETER2;

	switch(levelMode)
	{
		case 0: /* with value */
			 if (avq8384_invalidViInt16Range (levelUnit, 0, 3))
        		return VI_ERROR_PARAMETER4;
			Fmt (wrtBuf, "%s<REF %f%s", refLevel, LvlUntArr[levelUnit]);
			break;
		case 1: /* Peak */
		case 2: /* Cursor */		
			Fmt (wrtBuf, "%s<%s", levModeArr[(levelMode - 1)]);
			break;
		default: /* Min Max AUTO*/
		
			if (avq8384_invalidViBooleanRange (modeSwitch))
        	return VI_ERROR_PARAMETER5;

			Fmt (wrtBuf, "%s<%s %d",levModeArr[(levelMode - 1)], modeSwitch);     
	}

    total += NumFmtdBytes();

    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avq8384_status;

    return avq8384_status;    
}

/*===========================================================================*/
/* Function: Reference Level Scale
   This function sets Level Scale & Mode(Log/Linear).						 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_RefLvl_HldToClr (ViSession instrumentHandle,
                                          ViBoolean holdType)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;
    
    
	if (avq8384_invalidViBooleanRange (holdType))
   		return VI_ERROR_PARAMETER2;
   		
   if (holdType)
   {
		Fmt (wrtBuf, "%s<MXC"); 
   }
   else
   {
   		Fmt (wrtBuf, "%s<MNC");  
   }

    total += NumFmtdBytes();

    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avq8384_status;

    return avq8384_status;        
}

/*===========================================================================*/
/* Function: Reference Level Scale
   This function sets Level Scale & Mode(Log/Linear).						 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_RefLvlScl_MenuSet (ViSession instrumentHandle,
                                            ViBoolean display, ViInt16 scale)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;
    
    if (avq8384_invalidViBooleanRange (display))
        return VI_ERROR_PARAMETER2;
	if (avq8384_invalidViInt16Range (scale, 0, 6))
        return VI_ERROR_PARAMETER3;

	if (display == 1)	/* Linear */
	{
	 Fmt (wrtBuf, "%s<LIN %d",  display );
	}
	else
	{
	Fmt (wrtBuf, "%s<LIN %d; LEV %d", display, scale );
	 }
    total += NumFmtdBytes();

    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avq8384_status;

    return avq8384_status;    	    
}

/*===========================================================================*/
/* Function: Reference Level
   This function sets Level Value & Unit(Log/Linea).						 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_RefLvl_Set (ViSession instrumentHandle,
                                     ViReal64 refLevel, ViInt16 levelUnit)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;
    
	if (avq8384_invalidViInt16Range (levelUnit, 0, 3))
        return VI_ERROR_PARAMETER3;

	Fmt (wrtBuf, "%s<REF %f%s",refLevel, LvlUntArr[levelUnit]);

    total += NumFmtdBytes();

    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avq8384_status;

    return avq8384_status;	    
}

/*===========================================================================*/
/* Function: Average
   This function sets Average Count.										 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_FuncAvg_Set (ViSession instrumentHandle,
                                      ViInt16 averageMode, ViInt16 avgCount)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

    if (avq8384_invalidViInt16Range (averageMode, 0, 2))
        return VI_ERROR_PARAMETER2;

    if ( averageMode < 2 )
    {
		if (avq8384_invalidViInt16Range (avgCount, 1, 64))
        	return VI_ERROR_PARAMETER3;
	}
	else
	{
		if (((avgCount % 2) == 0) && (avq8384_invalidViInt16Range (avgCount, 1, 11)))  /* 1,3,5,7,9,11 are accepted. */
			return VI_ERROR_PARAMETER3; 
	}

	Fmt (wrtBuf, "%s<%s %d",avgModeArr[averageMode],avgCount);

    total += NumFmtdBytes();

    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avq8384_status;

    return avq8384_status;    
}

/*===========================================================================*/
/* Function: Resolution														 */
/* This function sets Resolution Value & Unit.								 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_FuncReso_Set (ViSession instrumentHandle,
                                       ViReal64 resolution,
                                       ViInt16 unit)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;
 
	if (avq8384_invalidViInt16Range (unit, 0,1))
        return VI_ERROR_PARAMETER3;

	Fmt (wrtBuf, "%s<RES %f%s",resolution, unitUmNmArr[unit]);

    total += NumFmtdBytes();


    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avq8384_status;

    return avq8384_status;     
}

/*===========================================================================*/
/* Function: Sweep Mode
   This function sets Sweep parameters.										 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_Swp_MenuSet (ViSession instrumentHandle,
                                      ViInt16 sweepMode, ViReal64 gateTime,
                                      ViInt16 gateTimeUnit, ViBoolean sync,
                                      ViReal64 delayTime, ViBoolean edge)
{

    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;
    
 	if (avq8384_invalidViInt16Range (sweepMode, 0,6))
        return VI_ERROR_PARAMETER2;   
	if (avq8384_invalidViInt16Range (gateTimeUnit, 0,1))
        return VI_ERROR_PARAMETER4;
	if (avq8384_invalidViBooleanRange (sync))
        return VI_ERROR_PARAMETER5;
    if (avq8384_invalidViReal64Range (delayTime, 0,1000))
        return VI_ERROR_PARAMETER6;
	if (avq8384_invalidViBooleanRange (edge))
        return VI_ERROR_PARAMETER7;
 
	Fmt (wrtBuf, "%s<SWE %d; PGT %f%s; SYN %d; DLY %f;EGE %d",sweepMode, gateTime, unitSecMsecArr[gateTimeUnit],sync,delayTime,edge);

    total += NumFmtdBytes();


    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avq8384_status;

    return avq8384_status;
}


/*===========================================================================*/
/* Function: Cursor Lambda1													 */
/* Purpose:  This function sets the parameters of Lobmda1.					 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_WavelenCurs1_Set (ViSession instrumentHandle,
                                           ViBoolean lambda1,
                                           ViBoolean xaxisMode,
                                           ViReal64 setLambda1,
                                           ViInt16 unitLambda1)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

    if (avq8384_invalidViBooleanRange (lambda1))    
        return VI_ERROR_PARAMETER2;
        
 	if ((lambda1 == VI_ON) ||(lambda1 == 1))
 	{
 		if (avq8384_invalidViBooleanRange (xaxisMode))    
        	return VI_ERROR_PARAMETER3;

 		if (avq8384_invalidViInt16Range (unitLambda1, 0, 1))
        	return VI_ERROR_PARAMETER5;

		if (xaxisMode)
		{
			Fmt (wrtBuf, "%s<FRQ%d; XAC 1; XAS %f%s", xaxisMode, setLambda1, unitGhzThzArr[unitLambda1]);     
		}
		else
		{
			Fmt (wrtBuf, "%s<FRQ%d; XAC 1; XAS %f%s", xaxisMode, setLambda1, unitUmNmArr[unitLambda1]);
		}
 	}
 	else
 	{
 		Fmt (wrtBuf, "%s<XAC 0");
 	}
 
 
    total += NumFmtdBytes();

    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avq8384_status;

    return avq8384_status;        
}

/*===========================================================================*/
/* Function: Cursor Lambda2													 */
/* Purpose:  This function sets the parameters of Lobmda2.					 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_Wavelen_Curs2 (ViSession instrumentHandle,
                                        ViBoolean lambda2, ViBoolean xaxisMode,
                                        ViReal64 setLambda2, ViInt16 unitLambda2)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

    if (avq8384_invalidViBooleanRange (lambda2))    
        return VI_ERROR_PARAMETER2;

 
 	if (lambda2 == VI_ON)
 	{
 		if (avq8384_invalidViBooleanRange (xaxisMode))    
        	return VI_ERROR_PARAMETER3;
 	 	if (avq8384_invalidViInt16Range (unitLambda2, 0, 1))
        	return VI_ERROR_PARAMETER5;
        
        if(xaxisMode)
		{

	 		Fmt (wrtBuf, "%s<FRQ %d; XBC 1; XBS %f%s", xaxisMode, setLambda2, unitGhzThzArr[unitLambda2]);
		}
		else
		{
			Fmt (wrtBuf, "%s<FRQ %d; XBC 1; XBS %f%s", xaxisMode, setLambda2, unitUmNmArr[unitLambda2]);
		}
 	}
 	else
 	{
 		Fmt (wrtBuf, "%s<XBC 0");
 	}

    total += NumFmtdBytes();

    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avq8384_status;

    return avq8384_status;        
    
}

/*===========================================================================*/
/* Function: Cursor Level1													 */
/* Purpose:  This function sets the parameters of Level1.					 */
/*===========================================================================*/

ViStatus CVIFUNC avq8384_LvlCurs1_Set (ViSession instrumentHandle,
                                       ViReal64 setLevel1, ViInt16 unitLevel1,
                                       ViBoolean level1)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

    if (avq8384_invalidViBooleanRange (level1))    
        return VI_ERROR_PARAMETER4;

 
 	if (level1 == VI_ON)
 	{
 	 if (avq8384_invalidViInt16Range (unitLevel1, 0,5))
        return VI_ERROR_PARAMETER3;

	 	Fmt (wrtBuf, "%s<YAC 1; YAS %f%s",setLevel1, unitLvlPerArr[unitLevel1]);
 	}
 	else{
 		Fmt (wrtBuf, "%s<YAC 0");
 	}
 
 
    total += NumFmtdBytes();

    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avq8384_status;

    return avq8384_status;       
}

/*===========================================================================*/
/* Function: Cursor Level2													 */
/* Purpose:  This function sets the parameters of Level3.					 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_LvlCurs2_Set (ViSession instrumentHandle,
                                       ViReal64 setLevel2, ViInt16 unitLevel2,
                                       ViBoolean level2)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

    if (avq8384_invalidViBooleanRange (level2))    
        return VI_ERROR_PARAMETER4;

 
 	if (level2 == VI_ON)
 	{
 	 if (avq8384_invalidViInt16Range (unitLevel2, 0,5))
        return VI_ERROR_PARAMETER3;

	 	Fmt (wrtBuf, "%s<YBC 1; YBS %f%s",setLevel2, unitLvlPerArr[unitLevel2]);
 	}
 	else{
 		Fmt (wrtBuf, "%s<YBC 0");
 	}
 
 
    total += NumFmtdBytes();

    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avq8384_status;

    return avq8384_status;       
}

/*===========================================================================*/
/* Function: Application Spec Width                                          */
/* Purpose:  This function sets the parameters of Spec Width.				 */
/*===========================================================================*/

ViStatus CVIFUNC avq8384_AppliSpcWid_MenuSet (ViSession instrumentHandle,
                                              ViBoolean specWidth,
                                              ViReal64 xdBParam,
                                              ViReal64 ydBParam, ViReal64 KParam,
                                              ViReal64 KRParam,
                                              ViReal64 xnmLevel,
                                              ViInt16 widthType,
                                              ViInt16 xnmLevelUnit)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

    if (avq8384_invalidViBooleanRange (specWidth))    
        return VI_ERROR_PARAMETER2;
 
 	if (specWidth)
 	{
    	if (avq8384_invalidViReal64Range (xdBParam, 0.1, 59.9))
        	return VI_ERROR_PARAMETER3;
    	if (avq8384_invalidViReal64Range (ydBParam, 0.1, 59.9))
        	return VI_ERROR_PARAMETER4;
    	if (avq8384_invalidViReal64Range (KParam, 0.1 , 100.0))
        	return VI_ERROR_PARAMETER5;
    	if (avq8384_invalidViReal64Range (KRParam, 1.0000, 10.0000))
        	return VI_ERROR_PARAMETER6;   
 		if (avq8384_invalidViInt16Range (widthType, 0,4))
        	return VI_ERROR_PARAMETER8;
 		if (avq8384_invalidViInt16Range (xnmLevelUnit, 0,1))
        	return VI_ERROR_INSTR_PARAMETER9;

    	Fmt (wrtBuf, "%s<WTY %d; WPX %f; WPY %f; WPK %f; WPR %f; WPW %f%s;SPW 1",
    		widthType, xdBParam, ydBParam, KParam, KRParam, xnmLevel,unitUmNmArr[xnmLevelUnit]);    
	}
	else
	{   	
		Fmt (wrtBuf, "%s<SPW 0");   
	}
   	total += NumFmtdBytes();

    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avq8384_status;

    return avq8384_status;     
}

/*===========================================================================*/
/* Function: Application Notch Width                                          */
/* Purpose:  This function sets the parameters of Notch Width.				 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_AppliNtchWid_MenuSet (ViSession instrumentHandle,
                                               ViInt16 notchWidth,
                                               ViReal64 level, ViReal64 width,
                                               ViInt16 widthUnit)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

 	if (avq8384_invalidViInt16Range (notchWidth, 0, 2))
        return VI_ERROR_PARAMETER2;
        
    if (notchWidth)
    {
		if (avq8384_invalidViReal64Range (level, 0.1, 59.9))
        	return VI_ERROR_PARAMETER3;

/*    	if (avq8384_invalidViReal64Range (width, 0.1, 59.9))
        	return VI_ERROR_PARAMETER4;  */
 	
 		if (avq8384_invalidViInt16Range (widthUnit, 0, 1))
        	return VI_ERROR_PARAMETER5;

    	Fmt (wrtBuf, "%s<NWD %d; NLV %f; NWI %f%s",
    		notchWidth, level, width, unitUmNmArr[widthUnit]);    
	}
	else
	{
		Fmt (wrtBuf, "%s<NWD 0");
	}
    
    total += NumFmtdBytes();
    
    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
      	return avq8384_status;

    return avq8384_status;     

}

/*===========================================================================*/
/* Function: Application WDM Oputical Amp                                    */
/* Purpose:  This function sets the parameters of Notch Width.				 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_AppliOptAmp_MenuSet (ViSession instrumentHandle,
                                              ViBoolean opticalAmp,
                                              ViBoolean opticalAmpMode,
                                              ViBoolean NFSelect,
                                              ViBoolean ASEFIT,
                                              ViBoolean spectrumDivision,
                                              ViReal64 KParameter2,
                                              ViReal64 maskedSpan,
                                              ViInt16 maskedSpanUnit,
                                              ViReal64 fittingSpan,
                                              ViInt16 fittingSpanUnit,
                                              ViReal64 filterDeltaL,
                                              ViInt16 filterDeltaLUnit,
                                              ViReal64 pinLoss,
                                              ViReal64 poutLoss,
                                              ViReal64 wdmYdBparam,
                                              ViBoolean WDMAseAuto)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

	if (avq8384_invalidViBooleanRange (opticalAmp))                                               
        return VI_ERROR_PARAMETER2;
        
    if ( opticalAmp == 1)
    {
		if (avq8384_invalidViBooleanRange (opticalAmpMode))                                               
        	return VI_ERROR_PARAMETER3;                                                           
		if (avq8384_invalidViBooleanRange (NFSelect))                                               
    	    return VI_ERROR_PARAMETER4;                                                           
		if (avq8384_invalidViBooleanRange (ASEFIT))                                               
    	    return VI_ERROR_PARAMETER5;                                                           
		if (avq8384_invalidViBooleanRange (spectrumDivision))                                               
        	return VI_ERROR_PARAMETER6;
		if (avq8384_invalidViReal64Range (KParameter2, 0.1, 100.0))                                               
        	return VI_ERROR_PARAMETER7;

/*		if (avq8384_invalidViReal64Range (maskedSpan, 0.00, 1100.00))
        	return VI_ERROR_PARAMETER8;
*/
		if (avq8384_invalidViInt16Range (maskedSpanUnit, 0, 1))
        	return VI_ERROR_INSTR_PARAMETER9;

/*		if (avq8384_invalidViReal64Range (fittingSpan, 0.00, 1100.00 ))
        	return VI_ERROR_INSTR_PARAMETER10;
*/
 		if (avq8384_invalidViInt16Range (fittingSpanUnit, 0, 1))
        	return VI_ERROR_INSTR_PARAMETER11;
        	
/*		if (avq8384_invalidViReal64Range (filterDeltaL, 0.00, 1100.00))
        	return VI_ERROR_INSTR_PARAMETER12;
*/
 		if (avq8384_invalidViInt16Range (filterDeltaLUnit, 0, 1))
        	return VI_ERROR_INSTR_PARAMETER13;
 		if (avq8384_invalidViReal64Range (pinLoss, -10.00, 10.00))
        	return VI_ERROR_INSTR_PARAMETER14;
		if (avq8384_invalidViReal64Range (poutLoss, -10.00, 10.00))
        	return VI_ERROR_INSTR_PARAMETER15;
		if (avq8384_invalidViReal64Range (wdmYdBparam, 0.1, 99.9))
        	return VI_ERROR_INSTR_PARAMETER16;
    	if ( avq8384_invalidViBooleanRange(WDMAseAuto) )
        	return VI_ERROR_INSTR_PARAMETER17;

    	Fmt (wrtBuf, "%s<OAM 0; OMD 0; WAU %d; NFT %d; FTM %d; SDV %d; NFK %f; SNA %f%s; SNB %f%s; FDL %f%s; LPI %f; LPO %f; WYD %f; OMD %d; OAM 1",
    		WDMAseAuto, NFSelect, 
    		ASEFIT, spectrumDivision, KParameter2, maskedSpan,
    		unitUmNmArr[maskedSpanUnit], fittingSpan,
    		unitUmNmArr[fittingSpanUnit], filterDeltaL,
    		unitUmNmArr[filterDeltaLUnit], pinLoss, poutLoss,
    		wdmYdBparam, opticalAmpMode);
     }
     else
     {
     	Fmt (wrtBuf, "%s<OAM 0");
     }
	
    total += NumFmtdBytes();
    
    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
      	return avq8384_status;

    return avq8384_status;     
    
}



/*===========================================================================*/
/* Function: Application WDM List                                    		*/
/* Purpose:  This function sets the parameters of WDM List.				 	*/
/*===========================================================================*/

ViStatus CVIFUNC avq8384_AppliWdmLst_MenuSet (ViSession instrumentHandle,
                                              ViBoolean WDMListDisp,
                                              ViInt16 WDMMode, ViBoolean listAll,
                                              ViReal64 WDMTresholdLevel)
{

    ViUInt32 retCnt = 0;                  
    ViUInt32 total = 0;  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;

	if (avq8384_invalidViBooleanRange (WDMListDisp))                                               
        return VI_ERROR_PARAMETER2;

    if (WDMListDisp == 1) /* LIST ON */
    {
    	if (avq8384_invalidViInt16Range (WDMMode, 0, 3))
        	return VI_ERROR_PARAMETER3;
        	
		if (avq8384_invalidViBooleanRange (listAll))                                               
        	return VI_ERROR_PARAMETER4;
        	
		if (avq8384_invalidViReal64Range (WDMTresholdLevel, 0.1, 99.9))                                               
        	return VI_ERROR_PARAMETER5;
        
        Fmt (wrtBuf, "%s<WDM 0; WMD %d; WAL %d; WYD %f; WDM 1",
     		 WDMMode, listAll, WDMTresholdLevel);

     }
     else
     {
     	Fmt (wrtBuf, "%s<WDM 0");	
     }
    total += NumFmtdBytes();
    
    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
      	return avq8384_status;

    return avq8384_status;     
   
}

/*===========================================================================*/
/* Function: Application WDM Ase Auto Set		                      		*/
/* Purpose:  This function sets the parameters of Ase Menu.  			 	*/
/*===========================================================================*/

ViStatus CVIFUNC avq8384_AppliWdmAse_Set (ViSession instrumentHandle,
                                          ViBoolean WDMAseAuto,
                                          ViInt16 aseFitting,
                                          ViReal64 manualAseLevel,
                                          ViInt16 levelUnit, 
                                          ViReal64 maskedSpan,
                                          ViReal64 fittingSpan)
{
    ViUInt32 retCnt = 0;                  
    ViUInt32 total = 0;  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;

	if (avq8384_invalidViBooleanRange (WDMAseAuto))                                               
        return VI_ERROR_PARAMETER2;
        
	Fmt (wrtBuf, "%s<WAU %d",WDMAseAuto);     
    total += NumFmtdBytes();
    
    if (WDMAseAuto == 0)  /* When ASE Auto Off, This control sets parameters */
    {
		if (avq8384_invalidViInt16Range (aseFitting, 0, 2))
        	return VI_ERROR_PARAMETER4;

		Fmt (wrtBuf, "%s[a]<; FTM %d",aseFitting);     
    		total += NumFmtdBytes();

		if (aseFitting == 2)
		{
	 		if (avq8384_invalidViInt16Range (levelUnit, 0, 3))
        		return VI_ERROR_PARAMETER5;

			if (avq8384_invalidViReal64Range ( maskedSpan, 0, 1100))
        		return VI_ERROR_PARAMETER4;	

			if (avq8384_invalidViReal64Range ( fittingSpan, 0, 1100))
        		return VI_ERROR_PARAMETER4;	

			Fmt (wrtBuf, "%s[a]<; MAL %f%s; SNA %fNM; SNB %fNM", 
				manualAseLevel,
				unitLvlPerArr[levelUnit],
				maskedSpan,
				fittingSpan);

			total += NumFmtdBytes();  
	 	}
	 }

    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
      	return avq8384_status;

    return avq8384_status;
}
											 
/*===========================================================================*/
/* Function: Application WDM Monitor Menu Set                        		*/
/* Purpose:  This function sets the parameters of WDM Monitor.			 	*/
/*===========================================================================*/
ViStatus CVIFUNC avq8384_AppliWdmMon_MenuSet (ViSession instrumentHandle,
                                              ViBoolean WDMMonitor,
                                              ViBoolean display,
                                              ViBoolean graphX, ViInt16 dataMode,
                                              ViInt16 currentSelectNo,
                                              ViInt16 measTime,
                                              ViReal64 timeInterval,
                                              ViReal64 WDMThresholdLevel)
{
	ViUInt32 retCnt = 0;                  
    ViUInt32 total = 0;  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;
	
	if (avq8384_invalidViBooleanRange (WDMMonitor))                                               
        return VI_ERROR_PARAMETER2;
        
    if (WDMMonitor)    
    {
		if (avq8384_invalidViBooleanRange (display))                                               
        	return VI_ERROR_PARAMETER3;

		if (avq8384_invalidViBooleanRange (graphX))                                               
        	return VI_ERROR_PARAMETER4;
        
    	if (avq8384_invalidViInt16Range (dataMode, 0, 2))
        	return VI_ERROR_PARAMETER5;
        
    	if (avq8384_invalidViInt16Range (measTime, 0, 501))
        	return VI_ERROR_PARAMETER7;
        
		if (avq8384_invalidViReal64Range (timeInterval, 0.1, 1440 ))
        	return VI_ERROR_PARAMETER8;	
        
		if (avq8384_invalidViReal64Range (WDMThresholdLevel, 0.1, 99.9 ))
        	return VI_ERROR_INSTR_PARAMETER9;
        
		Fmt (wrtBuf, "%s<LTS %d; LSD %d; LHA %d; LVA %d; ",WDMMonitor, display, graphX, dataMode);     
    	total += NumFmtdBytes();        
	
    	if (graphX)
    	{
    		Fmt (wrtBuf, "%s[a]<LCC%d; ", currentSelectNo);
    	}
    	else
    	{
    		Fmt (wrtBuf, "%s[a]<LCT%d; ", currentSelectNo);
    	}
    	total += NumFmtdBytes();
       
    	Fmt (wrtBuf, "%s[a]<LMT%d; LTI %f; WYD %f", measTime, timeInterval, WDMThresholdLevel);     
    }
    else
    {
    	Fmt (wrtBuf, "%s<LTS 0");	
    }
    
    
    
    total += NumFmtdBytes();        
    
    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
      	return avq8384_status;

    return avq8384_status;    
}

/*===========================================================================*/
/* Function: Application WDM Moniter pass Fail Set	                		 */
/* Purpose:  This function sets the parameters of Pass/Fail Menu.			 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_AppWdmMon_PfSet (ViSession instrumentHandle,
                                          ViBoolean limitJudgeenable,
                                          ViReal64 lambdaDriftLmt,
                                          ViReal64 levelUpperLimit,
                                          ViReal64 levelLowerLimit,
                                          ViInt16 levelLimitUnit,
                                          ViReal64 SNRLowerLimit)
{
	ViUInt32 retCnt = 0;                  
    ViUInt32 total = 0;  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;

	
    if (avq8384_invalidViBooleanRange (limitJudgeenable))
        return VI_ERROR_PARAMETER2;
        
	Fmt (wrtBuf, "%s<LTF %d",limitJudgeenable);	

	if (limitJudgeenable == 1)
	{
		if ( avq8384_invalidViInt16Range(levelLimitUnit,0,3))
        	return VI_ERROR_PARAMETER6;
	
		if (avq8384_invalidViReal64Range (SNRLowerLimit, 0.0 , 60.0 ))
        	return VI_ERROR_PARAMETER7;
		
		Fmt (wrtBuf, "%s[a]<; LFD %fNM; LUL %f%s; LLL %f%s; LSL %fDB",lambdaDriftLmt, levelUpperLimit, LvlUntArr[levelLimitUnit], levelLowerLimit, LvlUntArr[levelLimitUnit], SNRLowerLimit);

    	total += NumFmtdBytes();									   
	}
	
    total += NumFmtdBytes();        
    
    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
      	return avq8384_status;

    return avq8384_status;        
}

/*===========================================================================*/
/* Function: Application WDM Moniter ABS/REL		                  		 */
/* Purpose:  This function sets the parameters of ABS/REL Menu.			 	 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_AppWdmMon_ArSet (ViSession instrumentHandle,
                                          ViInt16 ABSREL, ViReal64 nominalLevel,
                                          ViInt16 nominalLevelUnit,
                                          ViReal64 nominalSNR)
{
	ViUInt32 retCnt = 0;                  
    ViUInt32 total = 0;  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;
	
    if (avq8384_invalidViInt16Range (ABSREL, 0, 2))
        return VI_ERROR_PARAMETER2;

	Fmt (wrtBuf, "%s<LAR %d",ABSREL);	
    
    total += NumFmtdBytes();        
	
	if (ABSREL == 2)
	{
    	if (avq8384_invalidViInt16Range (nominalLevelUnit, 0, 3))
        	return VI_ERROR_PARAMETER4;
        
        
		if (avq8384_invalidViReal64Range (nominalSNR, 0.0 , 60.0 ))
        	return VI_ERROR_PARAMETER5;
        
		Fmt (wrtBuf, "%s[a]<; LNL %f%s; LNS %f", nominalLevel, LvlUntArr[nominalLevelUnit], nominalSNR);	
    
    	total += NumFmtdBytes();        
    }
    
    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
      	return avq8384_status;

    return avq8384_status;    
}

/*===========================================================================*/
/* Function: Application Select Pin Menu Set                        		*/
/* Purpose:  This function sets the parameters of Select Pin.			 	*/
/*===========================================================================*/

ViStatus CVIFUNC avq8384_AppliSelPin_MenuSet (ViSession instrumentHandle,
                                              ViBoolean selectPin,
                                              ViReal64 levelValue,
                                              ViInt16 levelUnit,
                                              ViReal64 wavelengthValue,
                                              ViInt16 wvLenUnit)
{
	ViUInt32 retCnt = 0;                  
    ViUInt32 total = 0;  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;
	
	if (avq8384_invalidViBooleanRange (selectPin))                                               
        return VI_ERROR_PARAMETER2;
        
    if (avq8384_invalidViInt16Range (levelUnit, 0, 3))
        return VI_ERROR_PARAMETER4;
        
    if (avq8384_invalidViInt16Range (wvLenUnit, 0, 1))
        return VI_ERROR_PARAMETER6;

	if (selectPin == 1)
	{
		Fmt (wrtBuf, "%s<PLV %f%s; PLW %f%s; NPK 1",
     		 levelValue,
     		 LvlUntArr[levelUnit],
     		 wavelengthValue,
     		 unitUmNmArr[wvLenUnit]);
    }
    else
    {
    	Fmt (wrtBuf, "%s<NPK 0");	
    }
    total += NumFmtdBytes();
    
    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
      	return avq8384_status;

    return avq8384_status;     
       
}

/*===========================================================================*/
/* Function: Single Meas													 */
/* Purpose:  This function execute Single Measurement.						 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_SingleMeas_Start (ViSession instrumentHandle)
{
	ViUInt32 retCnt = 0;                  
	ViStatus avq8384_status = VI_SUCCESS;

	if ((avq8384_status = viWrite (instrumentHandle, "E", 1, &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;
}

/*===========================================================================*/
/* Function: Meas mode set & Execute										 */
/* Purpose:  This function execute Measurement mode.						 */
/*===========================================================================*/

ViStatus CVIFUNC avq8384_MeasMode_Start (ViSession instrumentHandle,
                                         ViInt16 measurementMode)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;

 	if (avq8384_invalidViInt16Range (measurementMode, 0, 2))  
	    return VI_ERROR_PARAMETER2;                      


	Fmt (wrtBuf, "%s<MEA %d",measurementMode);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;    
}
/*===========================================================================*/
/* Function: Meas Condition Setting											 */
/* Purpose:  This function sets the parameters for Measurement.				 */
/*===========================================================================*/

ViStatus CVIFUNC avq8384_MeasCond_MenuSet (ViSession instrumentHandle,
                                           ViInt16 average, ViInt16 sweepMode,
                                           ViReal64 resolution,
                                           ViInt16 resolutionUnit,
                                           ViInt16 samplingPoint)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;

 	if (avq8384_invalidViInt16Range (average, 1, 64))  
	    return VI_ERROR_PARAMETER2; 
 	if (avq8384_invalidViInt16Range (sweepMode, 0, 6))  
	    return VI_ERROR_PARAMETER3;
/*    if (avq8384_invalidViReal64Range (resolution, 0.0, 1.0))
        return VI_ERROR_PARAMETER4;		*/
 	if (avq8384_invalidViInt16Range (resolutionUnit, 0, 1))  
	    return VI_ERROR_PARAMETER5;                      
 	if (avq8384_invalidViInt16Range (samplingPoint, 0, 5))  
	    return VI_ERROR_PARAMETER6;                      


	Fmt (wrtBuf, "%s<AVG %d; SWE %d; RES %f%s; SPT %d",
		average,
		sweepMode,
		resolution,
		unitUmNmArr[resolutionUnit],
		samplingPoint);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;     
}

/*===========================================================================*/
/* Function: Auto Span Mode													 */
/* Purpose:  This function sets the parameters for Measurement.				 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_Func_AutoSpanMode (ViSession instrumentHandle,
                                            ViInt16 autoSpan)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;

 	if (avq8384_invalidViInt16Range (autoSpan, 0, 3))  
	    return VI_ERROR_PARAMETER2;                      


	Fmt (wrtBuf, "%s<AUT %d",
		autoSpan);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;        
}

/*===========================================================================*/
/* Function: Power Monitor Menu Set											 */
/* Purpose:  This function sets the parameters for Power Monitor.			 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_PowMoni_MenuSet (ViSession instrumentHandle,
                                          ViBoolean powerMoniter, 
                                          ViInt16 NMax,
                                          ViReal64 powerMonitorInterval)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;

    if (avq8384_invalidViBooleanRange (powerMoniter))
        return VI_ERROR_PARAMETER2;
 	if (avq8384_invalidViInt16Range (NMax, 11, 1001))  
	    return VI_ERROR_PARAMETER3;                      
	if (avq8384_invalidViReal64Range (powerMonitorInterval, 0.5, 3600.0))
        return VI_ERROR_PARAMETER4;	

	Fmt (wrtBuf, "%s<PMO %d; PNX %d; PIN %f",
		powerMoniter,NMax, powerMonitorInterval);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;            
}

/*===========================================================================*/
/* Function: Display Control Menu Set										 */
/* Purpose:  This function sets the parameters for Display.					 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_DspCtl_MenuSet (ViSession instrumentHandle,
                                         ViBoolean dual, ViBoolean grid,
                                         ViInt16 act, ViInt16 samplingPoint)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;
	
    if (avq8384_invalidViBooleanRange (dual))
        return VI_ERROR_PARAMETER2;

    if (avq8384_invalidViBooleanRange (grid))
        return VI_ERROR_PARAMETER3;
 	if (avq8384_invalidViInt16Range (act, 0, 1))  
	    return VI_ERROR_PARAMETER4;                      
	if (avq8384_invalidViInt16Range (samplingPoint, 0, 6))  
	    return VI_ERROR_PARAMETER5;                      


	Fmt (wrtBuf, "%s<DUA %d; GRI %d; AUL %d; APT %d",
		dual, grid, act, samplingPoint);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;
}

/*===========================================================================*/
/* Function: Display Control Multi Trace Menu								 */
/* Purpose:  This function sets the parameters of Multi Trace.				 */
/*===========================================================================*/

ViStatus CVIFUNC avq8384_DspCtrMlt_MenuSet (ViSession instrumentHandle,
                                            ViBoolean multiTrace,
                                            ViInt16 traceMax,
                                            ViInt16 currentTraceSet,
                                            ViBoolean autoTraceIncriment)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;
	
    if (avq8384_invalidViBooleanRange (multiTrace))
        return VI_ERROR_PARAMETER2;

	if( multiTrace == 0)
	{
		Fmt (wrtBuf, "%s<MTE 0");	
	}
	else
	{
    	if (avq8384_invalidViInt16Range (traceMax, 1, 32))
        	return VI_ERROR_PARAMETER3;
 		if (avq8384_invalidViInt16Range (currentTraceSet, 1, traceMax))  
	    	return VI_ERROR_PARAMETER4;                      
		if (avq8384_invalidViBooleanRange (autoTraceIncriment))  
	    	return VI_ERROR_PARAMETER5;                  

		Fmt (wrtBuf, "%s<MTE 1; MMX %d; MCU %d; MAT %d",traceMax, currentTraceSet, autoTraceIncriment);
	}

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;    
}

/*===========================================================================*/
/* Function: Display Control Multi Trace Control							 */
/* Purpose:  This function operate the data of Multi Trace.					 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_DspCtrMlt_TraceCtr (ViSession instrumentHandle,
                                             ViInt16 operation,
                                             ViInt16 traceNumber)
{
	ViUInt32 retCnt = 0;
	ViUInt32 total = 0;    	
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;

    if (avq8384_invalidViInt16Range (operation, 0, 4))
        return VI_ERROR_PARAMETER2;

	Fmt (wrtBuf, "%s<%s", mltTrCtlArr[operation]);

	total += NumFmtdBytes ();
	
	if(operation == 2) 
	{

 		if (avq8384_invalidViInt16Range (traceNumber, 1, 32))  
	    	return VI_ERROR_PARAMETER3;

		Fmt (wrtBuf, "%s[a]< %d", traceNumber);

		total += NumFmtdBytes ();

	}
	
	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;        
}

/*===========================================================================*/
/* Function: Super Inpose On Off											 */
/* Purpose:  This function switches Super Inpose.							 */
/*===========================================================================*/

ViStatus CVIFUNC avq8384_SpprImpose_OnOff (ViSession instrumentHandle,
                                           ViBoolean superImpose)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;
	

    if (avq8384_invalidViBooleanRange (superImpose))
        return VI_ERROR_PARAMETER2;
            
	Fmt (wrtBuf, "%s<SIM %d", superImpose);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;    
}

/*===========================================================================*/
/* Function: Cursor On Off													 */
/* Purpose:  This function switches Cursor.									 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_Cursor_OnOff (ViSession instrumentHandle,
                                       ViBoolean cursor)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;
	
    if (avq8384_invalidViBooleanRange (cursor))
        return VI_ERROR_PARAMETER2;

	Fmt (wrtBuf, "%s<CUR %d",cursor);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;    
}

/*===========================================================================*/
/* Function: Cursor Data Mode												 */
/* Purpose:  This function selects Cursor Mode.								 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_CursDatMode_Sel (ViSession instrumentHandle,
                                          ViInt16 dataMode)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;

	if (avq8384_invalidViInt16Range (dataMode, 0, 4))  
	    return VI_ERROR_PARAMETER2;                      

	Fmt (wrtBuf, "%s<CUD %d",dataMode);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;    
    
}
/*===========================================================================*/
/* Function: Lambda2 Set Next Peak											 */
/* Purpose:  This function sets Next Peak Lambda2.							 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_Lambda2NxtPk (ViSession instrumentHandle,
                                       ViBoolean nextLambda2)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;

	if (avq8384_invalidViBooleanRange (nextLambda2))  
	    return VI_ERROR_PARAMETER2;                      

	if (nextLambda2)   /* Left */
	{
		Fmt (wrtBuf, "%s<LPS");  
	}
	else			   /* Right */
	{
		Fmt (wrtBuf, "%s<RPS");  
	}

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;        
}

/*===========================================================================*/
/* Function: Peak Normalize													 */
/* Purpose:  This function switch Peak Normalize.							 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_NormPkNm_OnOff (ViSession instrumentHandle,
                                         ViBoolean peakNormalize)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;
	
    if (avq8384_invalidViBooleanRange (peakNormalize))
        return VI_ERROR_PARAMETER2;

	Fmt (wrtBuf, "%s<PNR %d",peakNormalize);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;    
}

/*===========================================================================*/
/* Function: Normalize Loss													 */
/* Purpose:  This function switch Loss mode.								 */
/*===========================================================================*/

ViStatus CVIFUNC avq8384_NormLoss_OnOff (ViSession instrumentHandle,
                                         ViBoolean loss)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;
	
    if (avq8384_invalidViBooleanRange (loss))
        return VI_ERROR_PARAMETER2;

	Fmt (wrtBuf, "%s<LOS %d",loss);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;        
}

/*===========================================================================*/
/* Function: Normalize Trans												 */
/* Purpose:  This function switch Trans mode.								 */
/*===========================================================================*/

ViStatus CVIFUNC avq8384_NormTrns_OnOff (ViSession instrumentHandle,
                                         ViBoolean trans)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;
	
    if (avq8384_invalidViBooleanRange (trans))
        return VI_ERROR_PARAMETER2;

	Fmt (wrtBuf, "%s<TRA %d",trans);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;     
}

/*===========================================================================*/
/* Function: Save Normalize													 */
/* Purpose:  This function execuute to save Normalise condition.			 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_Normalise_Save (ViSession instrumentHandle,
                                         ViInt16 saveMenu)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;
	
	if (avq8384_invalidViInt16Range (saveMenu, 0, 1))  
	    return VI_ERROR_PARAMETER3;
	
	 if (saveMenu == 1) /* MEAS 1 */
	 {
	 	Fmt (wrtBuf, "%s<SAV#MEAS1#");
	 }
	 else			   /* REF DATA */
	 {
	 	Fmt (wrtBuf, "%s<SAV#REF#");

	 }

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;     
}

/*===========================================================================*/
/* Function: Calib Lambda (Int.)											 */
/* Purpose:  This function sets the value of CAL Rambda(Int.).				 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_CalCalibInt_Start (ViSession instrumentHandle)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;

	if ((avq8384_status = viWrite (instrumentHandle, "CLM", 3, &retCnt)) < 0) 
	    return avq8384_status;   
    return avq8384_status;      
}
/*===========================================================================*/
/* Function: Calib Lambda (Ext.)											 */
/* Purpose:  This function sets the value of CAL Rambda(Ext.).				 */
/*===========================================================================*/

ViStatus CVIFUNC avq8384_CalCalibExt_Start (ViSession instrumentHandle,
                                            ViReal64 CALExt)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;
	
	if (avq8384_invalidViReal64Range (CALExt, 600.000, 1700.000))
        return VI_ERROR_PARAMETER2;	
	    
	Fmt (wrtBuf, "%s<CLE %fNM",CALExt);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;      
}

/*===========================================================================*/
/* Function: Calib Lambda Offset											 */
/* Purpose:  This function sets the value of Rambda Offset.					 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_CalLamdOfs_Set (ViSession instrumentHandle,
                                         ViReal64 lambdaOffset, ViInt16 unit)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;
	
	if (avq8384_invalidViInt16Range (unit, 0, 1))  
	    return VI_ERROR_PARAMETER3;
	    
	Fmt (wrtBuf, "%s<CLF %f%s",lambdaOffset,unitUmNmArr[unit]);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status; 
}

/*===========================================================================*/
/* Function: Calib Level Offset											 	 */
/* Purpose:  This function sets the value of Rambda Offset.					 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_CalLvlOfs_Set (ViSession instrumentHandle,
                                        ViReal64 levelOffset)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;
	
	if (avq8384_invalidViReal64Range (levelOffset, -20.000, 20.000))
        return VI_ERROR_PARAMETER2;
        
	Fmt (wrtBuf, "%s<CLS %fDB",levelOffset);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;    
}

/*===========================================================================*/
/* Function: Calibration AUTO Alignment.									 */
/* Purpose:  This function execute Auto Alignment.							 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_CalAutoAlign_Start (ViSession instrumentHandle)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;

	Fmt (wrtBuf, "%s<ALM");

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;    
}

/*===========================================================================*/
/* Function: Device Device type Select										 */
/* Purpose:  This function selects device type to output screen image(BMP).	 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_DevDevType_Sel (ViSession instrumentHandle,
                                         ViInt16 deviceType)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;

	if (avq8384_invalidViInt16Range (deviceType, 0, 2))  
	    return VI_ERROR_PARAMETER2;

	Fmt (wrtBuf, "%s<DEV %d", deviceType);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;    
}

/*===========================================================================*/
/* Function: Device Floppy On/Off	(Save/Recall Device)					 */
/* Purpose:  This function selects Save/Recall device type.					 */
/*					 ON : Floppy selected									 */
/*					 OFF: Memory Selected									 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_DevFlp_OnOff (ViSession instrumentHandle,
                                       ViBoolean floppy)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;

	if (avq8384_invalidViBooleanRange (floppy))
        return VI_ERROR_PARAMETER2;
        
	Fmt (wrtBuf, "%s<FON %d", floppy);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;    
    
}

/*===========================================================================*/
/* Function: Device Floppy Formatting										 */
/* Purpose: This function executes the initialization of 					 */
/*			the floppy disk by the selected format. 						 */
/*===========================================================================*/

ViStatus CVIFUNC avq8384_DevFlpFmt_Sel (ViSession instrumentHandle,
                                        ViInt16 floppyFormatting)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;

	if (avq8384_invalidViInt16Range (floppyFormatting, 1, 2))  
	    return VI_ERROR_PARAMETER2;
        
	Fmt (wrtBuf, "%s<FFO %d", floppyFormatting);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;    
}

/*===========================================================================*/
/* Function: Device Bitmap Menu set										 	*/
/* Purpose: This function set the mode of bitmap to output.					*/
/*===========================================================================*/
ViStatus CVIFUNC avq8384_DevBmp_MenuSet (ViSession instrumentHandle,
                                         ViInt16 saveImage,
                                         ViBoolean bitmapCompless)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;

	if (avq8384_invalidViInt16Range (saveImage, 0, 2))  
	    return VI_ERROR_PARAMETER2;
	if (avq8384_invalidViBooleanRange (bitmapCompless))
        return VI_ERROR_PARAMETER3;
                
	Fmt (wrtBuf, "%s<BCP %d; BSV %d", bitmapCompless, saveImage);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status; 	    
}

/*===========================================================================*/
/* Function: Device Color Pattern										 	*/
/* Purpose: This function set Color patern ID(0 to 4).						*/
/*===========================================================================*/
ViStatus CVIFUNC avq8384_DevColorPatt_Set (ViSession instrumentHandle,
                                           ViInt16 colorPattern)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;

	if (avq8384_invalidViInt16Range (colorPattern, 0, 4))  
	    return VI_ERROR_PARAMETER2;
                
	Fmt (wrtBuf, "%s<CPT %d", colorPattern);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;    
}

/*===========================================================================*/
/* Function: Device Color Pattern										 	*/
/* Purpose: This function set Color patern ID(0 to 4).						*/
/*===========================================================================*/
ViStatus CVIFUNC avq8384_DevExtPrt_MenuSet (ViSession instrumentHandle,
                                            ViInt16 extPrintMode,
                                            ViInt16 extPrintCommand)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;

	if (avq8384_invalidViInt16Range (extPrintMode, 0, 2))  
	    return VI_ERROR_PARAMETER2;
 	if (avq8384_invalidViInt16Range (extPrintCommand, 0, 2))  
	    return VI_ERROR_PARAMETER3;
                
               
	Fmt (wrtBuf, "%s<EPM %d; PRT %d", extPrintMode, extPrintCommand);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;    
}
/*===========================================================================*/
/* Function: Device Buzzer Menu Set										 	*/
/* Purpose: This function set the mode of Buzzer(BEEP).						*/
/*===========================================================================*/

ViStatus CVIFUNC avq8384_DevBuzzer_MenuSet (ViSession instrumentHandle,
                                            ViBoolean buzzer, 
                                            ViBoolean warning,
                                            ViBoolean quietBeep)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;

	if (avq8384_invalidViBooleanRange (buzzer))
        return VI_ERROR_PARAMETER2;
	if (avq8384_invalidViBooleanRange (warning))
        return VI_ERROR_PARAMETER3;
	if (avq8384_invalidViBooleanRange (quietBeep))
        return VI_ERROR_PARAMETER4;
        
                
	Fmt (wrtBuf, "%s<BUZ %d; WAR %d; QUI %d",buzzer, warning,quietBeep);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status; 	    
}

/*===========================================================================*/
/* Function: Device Clock Set											 	*/
/* Purpose: This function set the Data of Instrument's Clock.				*/
/*===========================================================================*/
ViStatus CVIFUNC avq8384_DevClock_Set (ViSession instrumentHandle, ViInt16 year,
                                       ViInt16 month, ViInt16 day, ViInt16 hour,
                                       ViInt16 minute)
{
	ViUInt32 retCnt = 0;
	ViUInt32 Total = 0;
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;
	ViChar yearS[3],monthS[3],dayS[3],hourS[3],minuteS[3];

	if (avq8384_invalidViInt16Range (year, 0, 99))  
	    return VI_ERROR_PARAMETER2;
 	if (avq8384_invalidViInt16Range (month, 1, 12))  
	    return VI_ERROR_PARAMETER3;
	if (avq8384_invalidViInt16Range (day, 1, 31))  
	    return VI_ERROR_PARAMETER4;
 	if (avq8384_invalidViInt16Range (hour, 0, 23))  
	    return VI_ERROR_PARAMETER5;
 	if (avq8384_invalidViInt16Range (minute, 0, 59))  
	    return VI_ERROR_PARAMETER6;
   
	if (year<10)
	{
		Fmt (yearS, "%s<0%d",year);         	
	}
	else
	{
		Fmt (yearS, "%s<%d",year);     
	}
	 
	if (month<10)
	{
		Fmt (monthS, "%s<0%d",month);         	
	}
	else
	{
		Fmt (monthS, "%s<%d",month);     
	}
	if (day<10)
	{
		Fmt (dayS, "%s<0%d",day);         	
	}
	else
	{
		Fmt (dayS, "%s<%d",day);     
	}
	 if (hour<10)
	{
		Fmt (hourS, "%s<0%d",hour);         	
	}
	else
	{
		Fmt (hourS, "%s<%d",hour);     
	}
	if (minute<10)
	{
		Fmt (minuteS, "%s<0%d",minute);         	
	}
	else
	{
		Fmt (minuteS, "%s<%d",minute);     
	}

	Fmt(wrtBuf, "%s<CLO #%s-%s-%s,%s:%s:00#",yearS,monthS,dayS,hourS,minuteS);


	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status; 	     
}

/*===========================================================================*/
/* Function: Device Clock ON/OFF										 	*/
/* Purpose: This function switches Clock Display.							*/
/*===========================================================================*/

ViStatus CVIFUNC avq8384_DevClock_OnOff (ViSession instrumentHandle,
                                         ViBoolean clock)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;


	if (avq8384_invalidViBooleanRange (clock))
        return VI_ERROR_PARAMETER2;

 	Fmt (wrtBuf, "%s<CKD %d",clock);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;    
}

/*===========================================================================*/
/* Function: Device Menu Output ON/OFF									 	*/
/* Purpose: This function switches output menu to Printer.					*/
/*===========================================================================*/
ViStatus CVIFUNC avq8384_DevMenuOut_OnOff (ViSession instrumentHandle,
                                           ViBoolean menuOutput)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;


	if (avq8384_invalidViBooleanRange (menuOutput))
        return VI_ERROR_PARAMETER2;

 	Fmt (wrtBuf, "%s<MEN %d",menuOutput);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;     
}

/*===========================================================================*/
/* Function: GPIB SRQ Signal ON/OFF										 	*/
/* Purpose: This function switches to output SRQ.							*/
/*===========================================================================*/
ViStatus CVIFUNC avq8384_SrqCtrl_OnOff (ViSession instrumentHandle,
                                         ViBoolean SRQOut)
{
 	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;


	if (avq8384_invalidViBooleanRange (SRQOut))
        return VI_ERROR_PARAMETER2;

 	Fmt (wrtBuf, "%s<SRQ %d",SRQOut);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;   
}

/*===========================================================================*/
/* Function: GPIB STB Mask Set											 	*/
/* Purpose: This function sets the mask of STB.								*/
/*===========================================================================*/
ViStatus CVIFUNC avq8384_StbMask_Set (ViSession instrumentHandle,
                                      ViInt16 maskValue)
{
 	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;

 	if (avq8384_invalidViInt16Range (maskValue, 0, 255))  
	    return VI_ERROR_PARAMETER2;


 	Fmt (wrtBuf, "%s<MSK %d", maskValue);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;	
}

/*===========================================================================*/
/* Function: GPIB STB Mask Clear											 */
/* Purpose: This function executes to clear the mask of STB.				 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_StbClear (ViSession instrumentHandle)
{
 	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;

 	Fmt (wrtBuf, "%s<CSB");

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;	    
}

/*===========================================================================*/
/* Function: GPIB Header Output ON/Off										 */
/* Purpose: This function switches to output Header(GPIB).					 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_HdData_OnOff (ViSession instrumentHandle,
                                       ViBoolean header)
{
 	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;
	
	if (avq8384_invalidViBooleanRange (header))
        return VI_ERROR_PARAMETER2;

 	Fmt (wrtBuf, "%s<HED %d", header);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;
}

/*===========================================================================*/
/* Function: GPIB Terminator Select											 */
/* Purpose:  This control selects the terminator.							 */
/*===========================================================================*/

ViStatus CVIFUNC avq8384_Terminator_Sel (ViSession instrumentHandle,
                                         ViInt16 terminator)
{
 	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;
	
	if (avq8384_invalidViInt16Range (terminator, 0, 3))  
	    return VI_ERROR_PARAMETER2;

 	Fmt (wrtBuf, "%s<DEL %d", terminator);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;
}

/*===========================================================================*/
/* Function: GPIB Data Separator Select										 */
/* Purpose:  This control selects the Separator Caractor.					 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_DataSepa_Sel (ViSession instrumentHandle,
                                       ViInt16 dataSeparator)
{
 	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;
	
	if (avq8384_invalidViInt16Range (dataSeparator, 0, 2))  
	    return VI_ERROR_PARAMETER2;

 	Fmt (wrtBuf, "%s<SDL %d", dataSeparator);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;
}

/*===========================================================================*/
/* Function: GPIB Message Separator Select									 */
/* Purpose:  This control selects the Separator Caractor.					 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_MessSepa_Sel (ViSession instrumentHandle,
                                       ViBoolean messageSeparator)
{
 	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;
	
	if (avq8384_invalidViBooleanRange (messageSeparator))
        return VI_ERROR_PARAMETER2;
        
 	Fmt (wrtBuf, "%s<MSP %d", messageSeparator);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;
    
}

/*===========================================================================*/
/* Function: Output  Data Select										 	*/
/* Purpose:  This control selects Data Type & Screen.						*/
/*===========================================================================*/

ViStatus CVIFUNC avq8384_DataOut_Sel (ViSession instrumentHandle,
                                      ViInt16 dataFormat, ViBoolean screenSelect)
{
  	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avq8384_status = VI_SUCCESS;
	
	if (avq8384_invalidViInt16Range (dataFormat, 0, 4))  
	    return VI_ERROR_PARAMETER2;
	if (avq8384_invalidViBooleanRange (screenSelect))
        return VI_ERROR_PARAMETER3;
        
 	Fmt (wrtBuf, "%s<FMT %d; OVS %d", dataFormat, screenSelect);

	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avq8384_status;                                                           
	                                                                                      
    return avq8384_status;
   
}



/*===========================================================================*/
/* Function: Write To Instrument                                             */
/* Purpose:  This function writes a command string to the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC CVIFUNC avq8384_writeInstrData (ViSession instrumentHandle,
                                         ViChar writeBuffer[])
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

    Fmt (wrtBuf, "%s<%s", writeBuffer);
    
    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avq8384_status;

    return avq8384_status;
}

/*===========================================================================*/
/* Function: Read Instrument Buffer                                          */
/* Purpose:  This function reads the output buffer of the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC avq8384_readInstrData (ViSession instrSession, ViInt32 numberBytesToRead,
                ViChar _VI_FAR readBuffer[], ViPInt32 numBytesRead)
{
    ViStatus avq8384_status = VI_SUCCESS;
    *numBytesRead = 0;
        
    if ((avq8384_status = viRead (instrSession, readBuffer, numberBytesToRead, numBytesRead)) < 0)
        return avq8384_status;

    return avq8384_status;
}

/*===========================================================================*/
/* Function: Meas Data Utility	                                         */
/* Purpose:  This function executes file Save/Load/Delete.			       */
/*===========================================================================*/

ViStatus CVIFUNC avq8384_MeasData (ViSession instrumentHandle, ViInt16 action,
                                   ViString filename)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[20];
    ViStatus avq8384_status = VI_SUCCESS;
    if (avq8384_invalidViInt16Range (action, 0, 2))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s#%s#",measDataActArr[action], filename);    
    total += NumFmtdBytes();
    
    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
      	return avq8384_status;

    return avq8384_status;     

        
}

/*===========================================================================*/
/* Function: Copy function copy		                                         */
/* Purpose:  This function executes print out.							       */
/*===========================================================================*/

ViStatus CVIFUNC avq8384_FuncCpoy_Start (ViSession instrumentHandle)
{
    ViUInt32 retCnt = 0;
    ViStatus avq8384_status = VI_SUCCESS;
     
    if ((avq8384_status = viWrite (instrumentHandle, "COP", 3, &retCnt)) < 0)
        return avq8384_status;

    return avq8384_status;   
}
/*===========================================================================*/
/* Function: Copy function feed		                                         */
/* Purpose:  This function executes internal print to feed.					 */
/*===========================================================================*/

ViStatus CVIFUNC avq8384_FuncCopy_Feed (ViSession instrumentHandle)
{
   ViUInt32 retCnt = 0;
    ViStatus avq8384_status = VI_SUCCESS;
     
    if ((avq8384_status = viWrite (instrumentHandle, "FEE", 3, &retCnt)) < 0)
        return avq8384_status;

    return avq8384_status;   
    
}

/*===========================================================================*/
/* Function: Reset                                                           */
/* Purpose:  This function resets the instrument.                            */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_reset (ViSession instrumentHandle, ViBoolean resetType)
{
    ViUInt32 retCnt = 0;
    ViStatus avq8384_status = VI_SUCCESS;

	 
	if (avq8384_invalidViBooleanRange (resetType))                                               
        return VI_ERROR_PARAMETER2;                                                           

	if (resetType)		
	{
		/* Initialize the instrument into the state when power saplay is turned on. */	
		if ((avq8384_status = viWrite (instrumentHandle, "C", 1, &retCnt)) < 0)
        	return avq8384_status;
	}
	else
	{
    	/* Initialize the instrument to a known state. */
    	if ((avq8384_status = viWrite (instrumentHandle, "IPR", 3, &retCnt)) < 0)
        	return avq8384_status;
	}
		
    if ((avq8384_status = avq8384_defaultInstrSetup (instrumentHandle)) < 0)  
        return avq8384_status;
        
    return avq8384_status;
}

/*===========================================================================*/
/* Function: Read Limit Line Data                                            */
/* Purpose:  This function read Limit line from floppy and select the state. */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_ReadLimLinData (ViSession instrumentHandle,
                                         ViBoolean readData, ViInt16 selectData)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[20];     
    ViStatus avq8384_status = VI_SUCCESS;

	if (avq8384_invalidViBooleanRange (readData))                                               
        return VI_ERROR_PARAMETER2;                                                           

    if (avq8384_invalidViInt16Range (selectData, 0, 5))
        return VI_ERROR_PARAMETER3;
     
    
    if (readData)     
    {
        if ((avq8384_status = viWrite (instrumentHandle, "LPR", 3, &retCnt)) < 0)
      	return avq8384_status;	
    }
    
     Fmt (wrtBuf, "%s<LPT %d", selectData);
     if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
      	return avq8384_status;
    
    return avq8384_status;   
}

/*===========================================================================*/
/* Function: Self Test	                                                     */
/* Purpose:  This function execute Self test.					             */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_SelfTest_Exe (ViSession instrumentHandle)
{
   ViUInt32 retCnt = 0;
   ViStatus avq8384_status = VI_SUCCESS;
   
   if ((avq8384_status = viWrite (instrumentHandle, "*TST", 4, &retCnt)) < 0)
   		return avq8384_status;
    
    return avq8384_status;      
   
}

/*===========================================================================*/
/* Function: Error Message                                                   */
/* Purpose:  This function translates the error return value from the        */
/*           instrument driver into a user-readable string.                  */
/*===========================================================================*/
ViStatus _VI_FUNC avq8384_errorMessage (ViSession instrSession, ViStatus statusCode,
                ViChar _VI_FAR errMessage[])
{
    ViStatus avq8384_status = VI_SUCCESS;
    ViInt16 i;
    static avq8384_tStringValPair statusDescArray[] = {
        {VI_WARN_NSUP_ID_QUERY,     "WARNING: ID Query not supported"},
        {VI_WARN_NSUP_RESET,        "WARNING: Reset not supported"},
        {VI_WARN_NSUP_SELF_TEST,    "WARNING: Self-test not supported"},
        {VI_WARN_NSUP_ERROR_QUERY,  "WARNING: Error Query not supported"},     
        {VI_WARN_NSUP_REV_QUERY,    "WARNING: Revision Query not supported"},
        {VI_ERROR_PARAMETER1,   "ERROR: Parameter 1 out of range"},
        {VI_ERROR_PARAMETER2,   "ERROR: Parameter 2 out of range"},
        {VI_ERROR_PARAMETER3,   "ERROR: Parameter 3 out of range"},
        {VI_ERROR_PARAMETER4,   "ERROR: Parameter 4 out of range"},
        {VI_ERROR_PARAMETER5,   "ERROR: Parameter 5 out of range"},
        {VI_ERROR_PARAMETER6,   "ERROR: Parameter 6 out of range"},
        {VI_ERROR_PARAMETER7,   "ERROR: Parameter 7 out of range"},
        {VI_ERROR_PARAMETER8,   "ERROR: Parameter 8 out of range"},
        {VI_ERROR_FAIL_ID_QUERY,"ERROR: Identification query failed"},
        {VI_ERROR_INV_RESPONSE, "ERROR: Interpreting instrument response"},
        {VI_ERROR_INSTR_FILE_OPEN,    "ERROR: Opening the specified file"},
        {VI_ERROR_INSTR_FILE_WRITE,   "ERROR: Writing to the specified file"},
        {VI_ERROR_INSTR_INTERPRETING_RESPONSE, "ERROR: Interpreting the instrument's response"},
        {VI_ERROR_INSTR_PARAMETER9 ,  "ERROR: Parameter 9 out of range"},
        {VI_ERROR_INSTR_PARAMETER10,  "ERROR: Parameter 10 out of range"},
        {VI_ERROR_INSTR_PARAMETER11,  "ERROR: Parameter 11 out of range"},
        {VI_ERROR_INSTR_PARAMETER12,  "ERROR: Parameter 12 out of range"},
        {VI_ERROR_INSTR_PARAMETER13,  "ERROR: Parameter 13 out of range"},
        {VI_ERROR_INSTR_PARAMETER14,  "ERROR: Parameter 14 out of range"},
        {VI_ERROR_INSTR_PARAMETER15,  "ERROR: Parameter 15 out of range"},
        {VI_ERROR_INSTR_PARAMETER16,  "ERROR: Parameter 16 out of range"},
        {avq8384_ERROR_INVALID_CONFIGURATION, "ERROR: Instrument configuration error"},
        
        {VI_NULL, VI_NULL}
    };

    avq8384_status = viStatusDesc (instrSession, statusCode, errMessage);
    if (avq8384_status == VI_WARN_UNKNOWN_STATUS) {
        for (i = 0; statusDescArray[i].stringName; i++) {
            if (statusDescArray[i].stringVal == statusCode) {
                Fmt (errMessage, "%s<%s", statusDescArray[i].stringName);
                return (VI_SUCCESS);
            }
        }
        Fmt (errMessage, "%s<Unknown Error 0x%x[uw8p0]", statusCode);
        return (VI_WARN_UNKNOWN_STATUS);
    }
    
    avq8384_status = VI_SUCCESS;
    return avq8384_status;
}

/***********
 ***********	Data Out Functions.
 ***********/

/*===========================================================================*/
/* Function: Waveform data                     			                 */
/* Purpose:  This function return point data array. 	                     */
/*===========================================================================*/

/* Short Data (16bit Only) */
ViStatus CVIFUNC avq8384_Waveform_Data (ViSession instrumentHandle,
                                        ViInt16 selecttype,
                                        short axisdataArray[])
{
    ViUInt32 retCnt = 0;
    ViInt16 formatdata, i;
    ViInt16	pointData;
    short *rdbuf_ptr;
    ViChar rdBuf[10010];
    ViChar wrtBuf[BUFFER_SIZE];      
    ViStatus avq8384_status = VI_SUCCESS;

    
	if (avq8384_invalidViInt16Range  (selecttype, 0, 3))                                               
        return VI_ERROR_PARAMETER2;                                                           
    
	/* Waveform Number */
	if ((avq8384_status = viWrite (instrumentHandle, "ODN?", 4, &retCnt)) < 0)        
        return avq8384_status;                                                    
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status; 
    if (Scan (rdBuf, "%s>ODN%d[b2]", &pointData) < 1)
    	if (Scan (rdBuf, "%s>%d[b2]", &pointData) < 1)  
		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;	

	/* Set data Binary */
    Fmt (wrtBuf, "%s<FMT1");   
    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)        
        return avq8384_status;
	

	/* Data read */
    Fmt (wrtBuf, "%s<OSD%d", selecttype);   
    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)        
        return avq8384_status;

    if ((avq8384_status = viRead (instrumentHandle, rdBuf, (pointData * 2), &retCnt)) < 0)
        	return avq8384_status;
        	
	Fmt(wrtBuf,"%s%d%s%d%s", "%",pointData,"d[zb2o10]>%",pointData,"d[b2]");
	if (Scan (rdBuf, wrtBuf,axisdataArray) < 1)
	    	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	
    return avq8384_status;
       
}


/*===========================================================================*/
/* Function: Hold Mode Data			                                         */
/* Purpose:  This function returns state of Hold Mode.						 */
/* NO Hold (0)/ Max Only (1)/ Min Only (2) Min & Max Hold(3)			     */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_holdMode_Data (ViSession instrumentHandle,
                                        ViInt16 *holdmode)
{
    ViUInt32 retCnt = 0;
    ViInt16 data;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;
	
	/* MAX? */
	if ((avq8384_status = viWrite (instrumentHandle, "MXH?", 4, &retCnt)) < 0)        
        return avq8384_status; 
        	
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status;

	if (Scan (rdBuf, "%s>MXH%d[b2]", &data) < 1)           	
	    if(Scan (rdBuf, "%s>%d[b2]", &data) < 1)   
			return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	*holdmode = data;

	/* MIN? */
	 
	 if ((avq8384_status = viWrite (instrumentHandle, "MNH?", 4, &retCnt)) < 0)        
        return avq8384_status; 
        	
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status;

	if (Scan (rdBuf, "%s>MNH%d[b2]", &data) < 1)           	
    	if(Scan (rdBuf, "%s>%d[b2]", &data) < 1)   
			return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	*holdmode += data * 2; 

    return avq8384_status; 
}

/*===========================================================================*/
/* Function: Data out Waveform Number                                      */
/* Purpose:  This function returns the samplimg point of waveform.         */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_WvDataNum_Data (ViSession instrumentHandle,
                                         ViInt16 *waveformDataNum)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;
    ViInt16 ret;

    if ((avq8384_status = viWrite (instrumentHandle, "SPT?", 4, &retCnt)) < 0)        
        return avq8384_status;                                                    
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status; 

   	if (Scan (rdBuf, "%s>SPT%d[b2]", &ret) < 1)
    	if (Scan (rdBuf, "%s>%d[b2]", &ret) < 1)
    		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    switch (ret)
 	{
 		case 0:
 			*waveformDataNum = 101;
 			break;
 		case 1:
 			*waveformDataNum = 201;
 			break;
 		case 2:
 			*waveformDataNum = 501;
 			break;
 		case 3:
 			*waveformDataNum = 1001;
 			break;
 		case 4:
 			*waveformDataNum = 2001;
 			break;
 		case 5:
 			*waveformDataNum = 5001;
 			break;
 		case 6:
 			*waveformDataNum = 10001;
 			break;		
 	}
 	
    return avq8384_status;
}

/*===========================================================================*/
/* Function: Waveform Data (ASCII)	                                         */
/* Purpose:  This function read ascii data and								 */
/*	returns the 64bit data array of waveform. 						         */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_Waveform_Data_As (ViSession instrumentHandle,
                                           ViInt16 selecttype,
                                           ViReal64 axisdataArray[])
{
    ViUInt32 retCnt = 0;
    ViUInt32 leng;
    ViInt16 num,i,head;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

	if ( avq8384_invalidViInt16Range(selecttype, 0, 3))
        return VI_ERROR_PARAMETER2;

	/* Header State? */
	if ((avq8384_status = viWrite (instrumentHandle, "HED?", 4, &retCnt)) < 0)        
        return avq8384_status;
	if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status;
        
   	if (Scan (rdBuf, "%s>%d[b2]", &head) < 1)

   	/* Header OFF */
	if ((avq8384_status = viWrite (instrumentHandle, "HED0", 4, &retCnt)) < 0)        
        return avq8384_status;

	/* Sampling Point*/
	if ((avq8384_WvDataNum_Data(instrumentHandle, &num))<0)
		return avq8384_status; 
	
    /* ASCII Set */ 
    if ((avq8384_status = viWrite (instrumentHandle, "FMT0", 4, &retCnt)) < 0)        
        return avq8384_status;   

    /* Output Data Read */
	if (selecttype == 1) /* X- Axis */
	{
		leng = 14;
		if ((avq8384_status = viWrite (instrumentHandle, "OSD1", 4, &retCnt)) < 0)        
        	return avq8384_status;    
	}	
	else		 /* Level & Min & Max */
	{
		leng = 12;
		if ((avq8384_status = viWrite (instrumentHandle, "OSD0", 4, &retCnt)) < 0)        
        	return avq8384_status;    
	}
				 
    for (i = 0;i < num;i++)
    {
    	if ((avq8384_status = viRead (instrumentHandle, rdBuf,leng, &retCnt)) < 0)
        	return avq8384_status;

   		if (Scan (rdBuf, "%s>%f",&axisdataArray[i]) < 1) 
			return VI_ERROR_INSTR_INTERPRETING_RESPONSE;	
     }

	if (head != 0)
	{
		if ((avq8384_status = viWrite (instrumentHandle, "HED1", 4, &retCnt)) < 0)        
        return avq8384_status;	
	}
	

    return avq8384_status;
        
}

/**************
 *********   Waveform Data Array Trance function. for CVI double & float format.
 *********   by Takahashi.
 ***************/


/* Binay String to Short Data Array */
void bin_ViInt16 (int size, char *src, ViInt16 *dst)
{
	unsigned char *pt_src;
	
	pt_src = (unsigned char *)src;
	
	while (--size>=0)
	{
		*dst    = *pt_src++ * 256;
		*dst++ += *pt_src++;
	}
}


/* Binay String to Double Data Array */
void bin_ViReal64 (int size, char *src, ViReal64 *dst)
{
	unsigned char buf[8];
	unsigned char *pt_src, *pt_dst;
	ViInt16       n;
	ViReal64      *dbl_src, *pt_buf;
	
	pt_buf  = (ViReal64 *)buf;
	dbl_src = (ViReal64 *)src;
	
	while (--size>=0)
	{
		pt_src = (unsigned char *)dbl_src + 7;
		pt_dst = buf;
		
		n = 8;
		while (--n>0)
			*pt_dst++ = *pt_src--;
		*pt_dst = *pt_src;	
			
		*dst++ = *pt_buf;
		dbl_src++;
	}
}


/* Binay String to Double Data Array */ 
void bin_float (int size, char *src, float *dst)
{
	unsigned char buf[4];
	unsigned char *pt_src, *pt_dst;
	ViInt16       n;
	float         *flt_src, *pt_buf;
	
	pt_buf  = (float *)buf;
	flt_src = (float *)src;
	
	while (--size>=0)
	{
		pt_src = (unsigned char *)flt_src + 3;
		pt_dst = buf;
		
		n = 4;
		while (--n>0)
			*pt_dst++ = *pt_src--;
		*pt_dst = *pt_src;	
			
		*dst++ = *pt_buf;
		flt_src++;
	}
}

/*===========================================================================*/
/* Function: Waveform Data (32bit)	                                         */
/* Purpose:  This function read 32bit data. 						         */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_Waveform_Data_32bit (ViSession instrumentHandle,
                                              ViInt16 selecttype,
                                              float axisdataArray[])
{
    ViUInt32 retCnt = 0;
    ViInt16 formatdata, i;
    ViInt16	pointData;
	float *rdbuf_ptr;
    unsigned char rdBuf[5001*4];
    unsigned char wrtBuf[BUFFER_SIZE];      
    ViStatus avq8384_status = VI_SUCCESS;

	if (avq8384_invalidViInt16Range  (selecttype, 0, 3))                                               
        return VI_ERROR_PARAMETER2;                                                           

	/* Sampling Point*/
	if ((avq8384_WvDataNum_Data(instrumentHandle, &pointData))<0)
		return avq8384_status; 

	/* Set data Binary 32bit*/
    Fmt (wrtBuf, "%s<FMT3");   
    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)        
        return avq8384_status;

	/* Data read */
    Fmt (wrtBuf, "%s<OSD%d", selecttype);   
    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)        
        return avq8384_status;

    if ((avq8384_status = viRead (instrumentHandle, rdBuf, (pointData * 4), &retCnt)) < 0)
        	return avq8384_status;

 
	bin_float (pointData, rdBuf, axisdataArray);


    return avq8384_status;	    
}

/*===========================================================================*/
/* Function: Waveform Data (64bit)	                                         */
/* Purpose:  This function read 64bit data. 						         */
/*===========================================================================*/

ViStatus CVIFUNC avq8384_Waveform_Data_64bit (ViSession instrumentHandle,
                                              ViInt16 selecttype,
                                              double axisdataArray[])
{
    ViUInt32 retCnt = 0;
    ViInt16 formatdata, i;
    ViInt16	pointData, head;
	ViChar *rdbuf_ptr;
    int temp;
    ViChar rdBuf[5001*8];
    ViChar wrtBuf[BUFFER_SIZE];      
    ViStatus avq8384_status = VI_SUCCESS;

	if (avq8384_invalidViInt16Range  (selecttype, 0, 3))                                               
        return VI_ERROR_PARAMETER2;                                                           

    
    
	/* Header State? */
	if ((avq8384_status = viWrite (instrumentHandle, "HED?", 4, &retCnt)) < 0)        
        return avq8384_status;
	if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status;

    if (Scan (rdBuf, "%s>HED%d[b2w1]", &head) < 1)         
   		if (Scan (rdBuf, "%s>%d[b2]", &head) < 1)
			return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
			  
   	/* Header OFF */
   	if (head == 1)
   	{
		if ((avq8384_status = viWrite (instrumentHandle, "HED0", 4, &retCnt)) < 0)        
        	return avq8384_status;
    } 
    
	/* Sampling Point*/
	if ((avq8384_WvDataNum_Data(instrumentHandle, &pointData))<0)
		return avq8384_status; 

	/* Set data Binary 64bit*/
    Fmt (wrtBuf, "%s<FMT2");   
    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)        
        return avq8384_status;

	/* Data read */
    Fmt (wrtBuf, "%s<OSD%d", selecttype);   

    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)        
        return avq8384_status;

    if ((avq8384_status = viRead (instrumentHandle, rdBuf, (pointData * 8), &retCnt)) < 0)
        return avq8384_status;

	bin_ViReal64 (pointData, rdBuf, axisdataArray);

    	/* Header OFF */
   	if (head == 1)
   	{
		if ((avq8384_status = viWrite (instrumentHandle, "HED1", 4, &retCnt)) < 0)        
        	return avq8384_status;
    } 
    return avq8384_status;    
}

/*===========================================================================*/
/* Function: Peak Search Data out	                                         */
/* Purpose:  This function returns the peak search data(Pwer Moniter).	     */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_PeakSrcPWR_Data (ViSession instrumentHandle,
                                          ViReal64 *min, ViReal64 *max,
                                          ViReal64 *average)
{
	ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;
	ViReal64 data[3];
	
    if ((avq8384_status = viWrite (instrumentHandle, "OPK?", 4, &retCnt)) < 0)        
        return avq8384_status; 
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status;

	if (Scan (rdBuf, "%s>LVMN%f,LVMI%f,LVAV%f", min, max, average )<3) 
		if (Scan (rdBuf, "%s>%f,%f,%f",min, max, average )<3)
			return VI_ERROR_INSTR_INTERPRETING_RESPONSE;   
 
    return avq8384_status;            
}


/*===========================================================================*/
/* Function: Peak Search Data out	                                         */
/* Purpose:  This function returns the peak search data(Spectrum).	     	 */
/*===========================================================================*/

ViStatus CVIFUNC avq8384_PeakSrcSpectrum_Data (ViSession instrumentHandle,
                                               ViReal64 *wavelength,
                                               ViReal64 *level)
{
    ViUInt32 retCnt = 0;
    ViInt16 head;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

	/* Header State? */
	if ((avq8384_status = viWrite (instrumentHandle, "HED?", 4, &retCnt)) < 0)        
        return avq8384_status;
	if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status;
        
   	if (Scan (rdBuf, "%s>%d[b2]", &head) < 1)

   	/* Header OFF */
	if ((avq8384_status = viWrite (instrumentHandle, "HED0", 4, &retCnt)) < 0)        
        return avq8384_status;

    if ((avq8384_status = viWrite (instrumentHandle, "OPK?", 4, &retCnt)) < 0)        
        return avq8384_status; 
        
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status;

/*    if (Scan (rdBuf, "%s>LMPK%f,LVPK%f", wavelength, level) < 2)
*/
	if (Scan (rdBuf, "%s>%f,%f", wavelength, level) < 2)
    		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
 	
	if (head != 0)
	{
		if ((avq8384_status = viWrite (instrumentHandle, "HED1", 4, &retCnt)) < 0)        
        return avq8384_status;	
	}


    return avq8384_status;        
}

/*===========================================================================*/
/* Function: Cursor Data out	                        		             */
/* Purpose:  This function returns cursor data.					             */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_Cursor_Data (ViSession instrumentHandle,
                                      ViBoolean *powerMode, ViInt16 *cursorMode,
                                      ViInt16 *holdMode,
                                      ViInt16 *trendDataNumber, ViReal64 *data1,
                                      ViReal64 *data2, ViReal64 *data3,
                                      ViReal64 *data4, ViReal64 *data5,
                                      ViReal64 *data6, ViReal64 *data7,
                                      ViReal64 *data8)
{
  	ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;
	ViInt16 headmode,cud;
	ViChar dust[10];

	/* Header? */
	if ((avq8384_status = viWrite (instrumentHandle, "HED?", 4, &retCnt)) < 0)        
        return avq8384_status;                                                    
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status; 
    if (Scan (rdBuf, "%s>HED%d[b2]", &headmode) < 1)       
    	 if (Scan (rdBuf, "%s>%d[b2]", &headmode) < 1)  
				return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	/* Header Off */
     if ((avq8384_status = viWrite (instrumentHandle, "HED0", 4, &retCnt)) < 0)        
        return avq8384_status;      
	
	/* Output: PMO?, CUD?, and data */
	
	/* Power Moniter ON? */
	if (avq8384_status = avq8384_PwrMoniMode_Data(instrumentHandle, powerMode))
		return avq8384_status; 
	
	if ( *powerMode == 1)
	{
		/* Power Moniter Result */

		if ((avq8384_status = viWrite (instrumentHandle, "OCD?", 4, &retCnt)) < 0)        
        		return avq8384_status; 
        
        /* Decimal + null + DS + Double Version */
        
        if ((avq8384_status = viRead (instrumentHandle, rdBuf, 5, &retCnt)) < 0)
        	return avq8384_status;	

 	
     	if(Scan (rdBuf, "%s>%d[w4b2]", trendDataNumber)<1)   
					return VI_ERROR_INSTR_INTERPRETING_RESPONSE;    
 		if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        	return avq8384_status;	

     		if(Scan (rdBuf, "%s>%f", data1)<1)   	
					return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	}
	else
	{
		/* Cursur Data Mode ? */
		if ((avq8384_status = viWrite (instrumentHandle, "CUD?", 4, &retCnt)) < 0)        
        	return avq8384_status; 
        	
    	if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        	return avq8384_status;
        	
     	if(Scan (rdBuf, "%s>%d[b2]", cursorMode)<1)   
			return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

		if( *cursorMode > 1 )
		{
			cud = *cursorMode;
			*holdMode = -1;
		}
		else
		{
			if (avq8384_status = avq8384_holdMode_Data(instrumentHandle, holdMode))
				return avq8384_status; 			

			if (*holdMode == 3) /* both hold On */
			{
				cud = *cursorMode + 5;	
			}
			else
			{
				cud = *cursorMode;
			}
		}
		
		if ((avq8384_status = viWrite (instrumentHandle, "OCD?", 4, &retCnt)) < 0)        
        		return avq8384_status; 
        		
        if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        	return avq8384_status;

		switch (cud)
		{
		case 0:
		case 1:
		case 4:
     		if(Scan (rdBuf, "%s>%f,%f,%f,%f,%f,%f", data1,data2,data3,data4,data5,data6)<6)   
				return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
			break;
		case 2:
     		if(Scan (rdBuf, "%s>%f,%f,%f,%f",data1,data2,data3,data4 )<4)   
				return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
			break;
		case 3:
     		if(Scan (rdBuf, "%s>%f,%f,%f,%f", data1,data2,data3)<3)   
				return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
			break;
		case 5:
			if(Scan (rdBuf, "%s>%f,%f,%f,%f,%f,%f,%f,%f", data1,data2,data3,data4,data5,data6,data7,data8)<8)   
				return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
			break;
		case 6:  /* 01.05.22 ver. */
			if(Scan (rdBuf, "%s>%f,%f,%f,%f,%f,%f%f%f", data1,data2,data3,data4,data5,data6,data7,data8)<8)   
				return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
			break;
		default:
			return VI_ERROR_PARAMETER7;
		}
	}

	if (headmode !=0)
	{
     if ((avq8384_status = viWrite (instrumentHandle, "HED1", 4, &retCnt)) < 0)        
        return avq8384_status;   	
	}

    return avq8384_status; 
}


/*===========================================================================*/
/* Function: Half Width Data out	                                        */
/* Purpose:  This function returns harf width data.		              */
/*===========================================================================*/

ViStatus CVIFUNC avq8384_Halfwidth_Data (ViSession instrumentHandle,
                                         ViReal64 *LMCN, ViReal64 *LMHW,
                                         ViInt16 *NOSP)
{
    ViUInt32 retCnt = 0;
	ViInt16 head;     
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;
    
	/* Header State? */
	if ((avq8384_status = viWrite (instrumentHandle, "HED?", 4, &retCnt)) < 0)        
        return avq8384_status;
	if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status;
        
   	if (Scan (rdBuf, "%s>%d[b2]", &head) < 1)

   	/* Header OFF */
	if ((avq8384_status = viWrite (instrumentHandle, "HED0", 4, &retCnt)) < 0)        
        return avq8384_status;
    
        
    if ((avq8384_status = viWrite (instrumentHandle, "OSW?", 4, &retCnt)) < 0)        
        return avq8384_status; 
        
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status;

 /* 	if (Scan (rdBuf, "%s>LMCN%f,LMHW%f,NOSP%d[w4b2]",  LMCN, LMHW, NOSP	) < 3)
  */
 	if (Scan (rdBuf, "%s>%f,%f,%d[w4b2]", LMCN, LMHW, NOSP) < 3)
    		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;


	if (head != 0)
	{
		if ((avq8384_status = viWrite (instrumentHandle, "HED1", 4, &retCnt)) < 0)        
        return avq8384_status;	
	}


    return avq8384_status;     
}
/*===========================================================================*/
/* Function: Notch Width Data out	                                        */
/* Purpose:  This function returns Notch width data.		              */
/*===========================================================================*/

ViStatus CVIFUNC avq8384_Notchwidth_Data (ViSession instrumentHandle,
                                          ViReal64 *LMCN, ViReal64 *LMHW)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;
 	ViInt16 head;

	/* Header State? */
	if ((avq8384_status = viWrite (instrumentHandle, "HED?", 4, &retCnt)) < 0)        
        return avq8384_status;
	if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status;
        
   	if (Scan (rdBuf, "%s>%d[b2]", &head) < 1)

   	/* Header OFF */
	if ((avq8384_status = viWrite (instrumentHandle, "HED0", 4, &retCnt)) < 0)        
        return avq8384_status;



    if ((avq8384_status = viWrite (instrumentHandle, "ONW?", 4, &retCnt)) < 0)        
        return avq8384_status; 
        
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status;

/*    if (Scan (rdBuf, "%s>LMCN%f,LMHW%f",  LMCN,LMHW) < 2)
 */
 	if (Scan (rdBuf, "%s>%f,%f", LMCN,LMHW) < 2)
    		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	/* Header ON */
	if (head != 0)
	{
		if ((avq8384_status = viWrite (instrumentHandle, "HED1", 4, &retCnt)) < 0)        
        return avq8384_status;	
	}

    return avq8384_status;    
}


/*===========================================================================*/
/* Function: Gain & NF Data out	                                        	*/
/* Purpose:  This function returns Gain & NF data.			              	*/
/*===========================================================================*/

ViStatus CVIFUNC avq8384_GainNf_Data (ViSession instrumentHandle, ViReal64 *LMCN,
                                      ViReal64 *GAIN, ViReal64 *PASE,
                                      ViReal64 *NF)
{
    ViUInt32 retCnt = 0;
	ViInt16 head;    
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;


	/* Header State? */
	if ((avq8384_status = viWrite (instrumentHandle, "HED?", 4, &retCnt)) < 0)        
        return avq8384_status;
	if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status;
   	if (Scan (rdBuf, "%s>%d[b2]", &head) < 1)

   	/* Header OFF */
	if (head != 0)
	{	
		if ((avq8384_status = viWrite (instrumentHandle, "HED0", 4, &retCnt)) < 0)        
        	return avq8384_status;
	}

	/* DATA? */
    if ((avq8384_status = viWrite (instrumentHandle, "OGN?", 4, &retCnt)) < 0)        
        return avq8384_status; 
        
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status;

 /*   if (Scan (rdBuf, "%s>LMCN%f,GAIN%f,PASE%f,NF%f",  LMCN,GAIN,PASE,NF) < 4)
  */
  	
  	if (Scan (rdBuf, "%s>%f,%f,%f,%f", LMCN,GAIN,PASE,NF) < 4)
    		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	/* Header ON */
	if (head != 0)
	{
		if ((avq8384_status = viWrite (instrumentHandle, "HED1", 4, &retCnt)) < 0)        
        return avq8384_status;	
	}

    return avq8384_status;    
}  
/*===========================================================================*/
/* Function: OptAmp WDM List Data 1			 								 */
/* Purpose:  This function returns the result of OptAmp gain 				 */
/*   noise.						 											 */
/*===========================================================================*/

ViStatus CVIFUNC avq8384_OptAmpWdmResult1_Data (ViSession instrumentHandle,
                                                ViReal64 lambdaArr[],
                                                ViReal64 gainArr[],
                                                ViReal64 paseArr[],
                                                ViReal64 NFArr[],
                                                ViInt16 *peakCount)
{
	ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;
    ViInt16 i,head;
    
	/* Header */
	if ((avq8384_status = viWrite (instrumentHandle, "HED?", 4, &retCnt)) < 0)        
        return avq8384_status;
	if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status;
        
   	if (Scan (rdBuf, "%s>HED%d[b2]", &head) < 1)
   		if (Scan (rdBuf, "%s>%d[b2]", &head) < 1)
        	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;  		

	if(head == 1)
	{
		if ((avq8384_status = viWrite (instrumentHandle, "HED0", 4, &retCnt)) < 0)        
        	return avq8384_status;
	}

	/* Count? */
	if ((avq8384_status = viWrite (instrumentHandle, "OWP?", 4, &retCnt)) < 0)        
        return avq8384_status;                                                    
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status; 

   	if (Scan (rdBuf, "%s>%d[b2]", peakCount) < 1)
    		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;  		

  	if (*peakCount > 0)
  	{
  		if ((avq8384_status = viWrite (instrumentHandle, "OLS?", 4, &retCnt)) < 0)
        	return avq8384_status;	

  		for (i = 0 ; i < *peakCount; i++)
  		{
  			if ((avq8384_status = viRead (instrumentHandle, rdBuf, 50, &retCnt)) < 0)
        		return avq8384_status; 
			if(avq8384_status != VI_SUCCESS_MAX_CNT)
			{
				break;
			}
			avq8384_status = 0;
			
   			if (Scan (rdBuf, "%s>%f,%f,%f,%f",
					&lambdaArr[i],
                    &gainArr[i],
                    &paseArr[i],
                    &NFArr[i]   			
   					) < 4)
    			return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		}
	}

	if (head != 0)
	{
		if ((avq8384_status = viWrite (instrumentHandle, "HED1", 4, &retCnt)) < 0)        
        return avq8384_status;	
	}

    return avq8384_status;
}

/*===========================================================================*/
/* Function: OptAmp WDM List Data 2			 								 */
/* Purpose:  This function returns the result of OptAmp gain 				 */
/*   noise and Pout.			 											 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_OptAmpWdmResult2_Data (ViSession instrumentHandle,
                                                ViReal64 lambdaArr[],
                                                ViReal64 poutArr[],
                                                ViReal64 gainArr[],
                                                ViReal64 paseArr[],
                                                ViReal64 NFArr[],
                                                ViInt16 *peakCount)
{
	ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;
    ViInt16 i,head;
    
	/* Header */
	if ((avq8384_status = viWrite (instrumentHandle, "HED?", 4, &retCnt)) < 0)        
        return avq8384_status;
	if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status;
        
   	if (Scan (rdBuf, "%s>HED%d[b2]", &head) < 1)
   		if (Scan (rdBuf, "%s>%d[b2]", &head) < 1)
        	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;  		

	if(head == 1)
	{
		if ((avq8384_status = viWrite (instrumentHandle, "HED0", 4, &retCnt)) < 0)        
        	return avq8384_status;
	}

	/* Count? */
	if ((avq8384_status = viWrite (instrumentHandle, "OWP?", 4, &retCnt)) < 0)        
        return avq8384_status;                                                    
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status; 

   	if (Scan (rdBuf, "%s>%d[b2]", peakCount) < 1)
    		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;  		

  	if (*peakCount > 0)
  	{
  		if ((avq8384_status = viWrite (instrumentHandle, "OWN?", 4, &retCnt)) < 0)
        	return avq8384_status;	

  		for (i = 0 ; i < *peakCount; i++)
  		{
  			if ((avq8384_status = viRead (instrumentHandle, rdBuf, 62, &retCnt)) < 0)
        		return avq8384_status; 
			if(avq8384_status != VI_SUCCESS_MAX_CNT)
			{
				break;
			}
			avq8384_status = 0;
			
   			if (Scan (rdBuf, "%s>%f,%f,%f,%f,%f",
					&lambdaArr[i],
                    &poutArr[i],
                    &gainArr[i],
                    &paseArr[i],
                    &NFArr[i]   			
   					) < 5)
    			return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		}
	}

	if (head != 0)
	{
		if ((avq8384_status = viWrite (instrumentHandle, "HED1", 4, &retCnt)) < 0)        
        return avq8384_status;	
	}

    return avq8384_status;        
    
}

/*===========================================================================*/
/* Function: Power Monitor Data out	                                         */
/* Purpose:  This function returns Power Monitor data.			             */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_PowerMoni_Data (ViSession instrumentHandle,
                                         ViReal64 *LVLG)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

    if ((avq8384_status = viWrite (instrumentHandle, "OPM?", 4, &retCnt)) < 0)        
        return avq8384_status; 
        
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status;

    if (Scan (rdBuf, "%s>LVLG%f",  LVLG) < 1)
    	if (Scan (rdBuf, "%s>%f", LVLG) < 1)
    		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avq8384_status;    
}

/*===========================================================================*/
/* Function: Power Monitor N Max Data		                                 */
/* Purpose:  This function returns the point of Trend-Chart.			     */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_PowerMoniNmax_Data (ViSession instrumentHandle,
                                             ViInt16 *nmax)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

    if ((avq8384_status = viWrite (instrumentHandle, "PNX?", 4, &retCnt)) < 0)        
        return avq8384_status; 
        
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status;

	if (Scan (rdBuf, "%s>PNX%d[b2]",  nmax) < 1)       
    	if (Scan (rdBuf, "%s>%d[b2]", nmax) < 1)
    		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
    return avq8384_status;     
}

/*===========================================================================*/
/* Function: Power Monitor On Off Data out	                                 */
/* Purpose:  This function returns Power Monitor On/Off data.			     */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_PwrMoniMode_Data (ViSession instrumentHandle,
                                           ViInt16 *mode)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

    if ((avq8384_status = viWrite (instrumentHandle, "PMO?", 4, &retCnt)) < 0)        
        return avq8384_status; 
        
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status;
																		
	if (Scan (rdBuf, "%s>PMO%d[b2]",  mode) < 1)
 		if (Scan (rdBuf, "%s>%d[b2]", mode) < 1)
    		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avq8384_status;        
}

/*===========================================================================*/
/* Function: GAIN NOISE INDEX TOTAL ASE POWER OUT DEMAND Data out             */
/* Purpose:  This function returns GAIN NOISE INDEX TOTAL ASE POWER OUT DEMAND */
/*				data.											              */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_GainNfTotAse_Data (ViSession instrumentHandle,
                                            ViReal64 *LMCN, ViReal64 *GAIN,
                                            ViReal64 *PASE, ViReal64 *NF,
                                            ViReal64 *PSPW)
{
    ViUInt32 retCnt = 0;
	ViInt16 head;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;


	/* Header State? */
	if ((avq8384_status = viWrite (instrumentHandle, "HED?", 4, &retCnt)) < 0)        
        return avq8384_status;
	if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status;

   	if (Scan (rdBuf, "%s>HED%d[b2]", &head) < 1)
   		if (Scan (rdBuf, "%s>%d[b2]", &head) < 1)
        	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;  		

	if(head == 1)
	{
		if ((avq8384_status = viWrite (instrumentHandle, "HED0", 4, &retCnt)) < 0)        
        	return avq8384_status;
	} 


    if ((avq8384_status = viWrite (instrumentHandle, "OPN?", 4, &retCnt)) < 0)        
        return avq8384_status; 
        
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status;

 /*   if (Scan (rdBuf, "%s>LMCN%f,GAIN%f,PASE%f,NF%f,PSPW%f",  LMCN, GAIN, PASE, NF, PSPW) < 5)
  */
  
  	if (Scan (rdBuf, "%s>%f,%f,%f,%f,%f", LMCN, GAIN, PASE, NF, PSPW) < 5)
    		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

  	/* Header ON */
	if (head != 0)
	{
		if ((avq8384_status = viWrite (instrumentHandle, "HED1", 4, &retCnt)) < 0)        
        return avq8384_status;	
	}

    return avq8384_status;     
}


/*===========================================================================*/
/* Function: WDM Peak Number Data         									*/
/* Purpose:  This function returns WDM Peak Number							 */
/*===========================================================================*/

ViStatus CVIFUNC avq8384_WdmPkNum_Data (ViSession instrumentHandle,
                                        ViInt16 *WDMPeakNum)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

    if ((avq8384_status = viWrite (instrumentHandle, "OWP?", 4, &retCnt)) < 0)        
        return avq8384_status;                                                    
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status; 

   	if (Scan (rdBuf, "%s>OWP%d[b2]", WDMPeakNum) < 1)
    	if (Scan (rdBuf, "%s>%d[b2]", WDMPeakNum) < 1)
    		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
  		
    return avq8384_status;
    
}
/*===========================================================================*/
/* Function: WDM List On Off Data            									*/
/* Purpose:  This function returns the state of WDM list.							 */
/*===========================================================================*/

ViStatus CVIFUNC avq8384_WdmListOnOff_Data (ViSession instrumentHandle,
                                            ViBoolean *WDMListState)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

    if ((avq8384_status = viWrite (instrumentHandle, "WDM?", 4, &retCnt)) < 0)        
        return avq8384_status;                                                    
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status; 

   	if (Scan (rdBuf, "%s>WDM%d[b2]", WDMListState) < 1)
    	if (Scan (rdBuf, "%s>%d[b2]", WDMListState) < 1)
    		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
  		
    return avq8384_status;   
}

/*===========================================================================*/
/* Function: List Data Number Data            									*/
/* Purpose:  This function returns Number of list Data.							 */
/*===========================================================================*/

ViStatus CVIFUNC avq8384_ListDatNum_Data (ViSession instrumentHandle,
                                          ViInt16 *listDataNumber)
{
   ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

    if ((avq8384_status = viWrite (instrumentHandle, "OLN?", 4, &retCnt)) < 0)        
        return avq8384_status;                                                    
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status; 

   	if (Scan (rdBuf, "%s>OLN%d[b2]", listDataNumber) < 1)
    	if (Scan (rdBuf, "%s>%d[b2]", listDataNumber) < 1)
    		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
  		
    return avq8384_status;
    
}

/*===========================================================================*/
/* Function: List DataData            									*/
/* Purpose:  This function returns list Data							 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_WdmListOn_Data (ViSession instrumentHandle,
                                         ViInt16 *WDMMode, ViInt16 *listNum,
                                         ViReal64 array1[], ViReal64 array2[],
                                         ViReal64 array3[], ViReal64 array4[],
                                         ViReal64 array5[])
{
    ViUInt32 readSet;
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE],buf[50];
    ViStatus avq8384_status = VI_SUCCESS;
    ViInt16 head,onOff,n,i,datanum;
    

    
    /* WDM List On? */
	if ((avq8384_status =avq8384_WdmListOnOff_Data(instrumentHandle, &onOff)) < 0)        
        return avq8384_status;                                                    
    
    if (onOff == 0)
    {
		/* No reaction */
		*listNum = 0;
		*WDMMode = -1;
		return avq8384_status;    
    }
    else /* List On */
    {
    
    /* Header? */
    
    if ((avq8384_status = viWrite (instrumentHandle, "HED?", 4, &retCnt)) < 0)        
        return avq8384_status;                                                    
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status; 
    if (Scan (rdBuf, "%s>HED%d[b2]", &head) < 1)       
    	 if (Scan (rdBuf, "%s>%d[b2]", &head) < 1)  
				return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
    /*Header Off*/

	if (head == 1)
	{
    	if ((avq8384_status = viWrite (instrumentHandle, "HED0", 4, &retCnt)) < 0)        
        	return avq8384_status;    
    }
    
	/* Mode? */

	if ((avq8384_status = viWrite (instrumentHandle, "WMD?", 4, &retCnt)) < 0)        
        return avq8384_status;                                                    
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status; 

   	if (Scan (rdBuf, "%s>%d[b2]", WDMMode) < 1)
    		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	}
	
	/* List Number data */
	if ((avq8384_status = avq8384_ListDatNum_Data(instrumentHandle,listNum) )< 0)        
        return avq8384_status;    
	if (*listNum > 0)
	{

    	if ((avq8384_status = viWrite (instrumentHandle, "OLS?", 4, &retCnt)) < 0)        
        	return avq8384_status;                                                    


		 /* Mode 0	 	Malti Peak: 	double * 2  1set 26
		 	Mode 1		SNR:		 	double * 4  1set 50
		 	Mode 2		Relative:		double * 5 14,14,14,12,12 = 1set 66  
		 	Mode 3		ITU-GRID:		double * 4 54?
		 */

		switch (*WDMMode)
		{
			case 0:
				datanum = 2;
				Fmt(buf,"%s<%s","%s>%f,%f"); 
				readSet = 26;
				break; 
			case 1:
				datanum = 4;
				Fmt(buf,"%s<%s","%s>%f,%f,%f,%f");
				readSet = 50;
			
				break; 
			case 2:
				datanum = 5;
				Fmt(buf,"%s<%s","%s>%f,%f,%f,%f,%f");
				readSet = 66;
			   break;
			case 3:
				datanum = 4;
				Fmt(buf,"%s<%s","%s>%f,%f,%f,%f");
				readSet = 54;
		}

		for (i = 0;i < *listNum; i++)
		{
		
    		if ((avq8384_status = viRead (instrumentHandle, rdBuf, readSet , &retCnt)) < 0)
        		return avq8384_status; 

			switch ( *WDMMode)
			{
			case 0:
				if (Scan (rdBuf, buf, array1 , array2) < 2)
    				return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    			array1++;
    			array2++;
				break;
			case 1:
			case 3: 
				if (Scan (rdBuf, buf, array1 , array2, array3,array4) < 4)     
    				return VI_ERROR_INSTR_INTERPRETING_RESPONSE; 
    			array1++;
    			array2++;
    			array3++;
    			array4++;
				break;
			case 2:
				if (Scan (rdBuf, buf,array1 , array2, array3, array4, array5 ) < 5)
    				return VI_ERROR_INSTR_INTERPRETING_RESPONSE; 		
				array1++;
    			array2++;
    			array3++;
    			array4++;
				array5++;
				break;
			}
		}
	}
	
	/* if listnum > 0 end */
    Fmt(buf,"HED%d",head);   
    if ((avq8384_status = viWrite (instrumentHandle, buf, NumFmtdBytes(), &retCnt)) < 0)        
        return avq8384_status;    
  		
    return avq8384_status;
    
}

/*===========================================================================*/
/* Function: WDM Monitor Data			            						*/
/* Purpose:  This function returns the data of WDM Monitor 					*/
/*===========================================================================*/

ViStatus CVIFUNC avq8384_WdmMon_Data (ViSession instrumentHandle,
                                      ViInt16 timeData, ViInt16 *passFail,
                                      ViInt16 *peakCount, ViReal64 XData[],
                                      ViReal64 level[], ViReal64 SNR[],
                                      ViReal64 XJugde[], ViReal64 LJudge[],
                                      ViReal64 SJudge[])
{
	ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViChar wrtBuf[BUFFER_SIZE];      
    ViStatus avq8384_status = VI_SUCCESS;
    ViInt16 i,head;
	/* Header */


	if ((avq8384_status = viWrite (instrumentHandle, "HED?", 4, &retCnt)) < 0)        
        return avq8384_status;
	if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status;
        
   	if (Scan (rdBuf, "%s>%d[b2]", &head) < 1)
   
	if ((avq8384_status = viWrite (instrumentHandle, "HED0", 4, &retCnt)) < 0)        
        return avq8384_status;

	/* Pass or Fail Enable? */
    if ((avq8384_status = viWrite (instrumentHandle, "LTF?", 4, &retCnt)) < 0)        
        return avq8384_status;                                                    
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status; 

   	if (Scan (rdBuf, "%s>%d[b2]", passFail) < 1)
    		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	/* Count? */
	if ((avq8384_status = viWrite (instrumentHandle, "OLN?", 4, &retCnt)) < 0)        
        return avq8384_status;                                                    
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status; 

   	if (Scan (rdBuf, "%s>%d[b2]", peakCount) < 1)
    		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;  		
  		
  	/* WDM LIST DATA */
  	Fmt (wrtBuf, "%s<OLT%d[w3p0]",timeData);         
  	if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avq8384_status;	
  	
    if (*passFail)  /* judge ON */
  	{
  	
  		for (i = 0 ; i < *peakCount; i++)
  		{
  			if ((avq8384_status = viRead (instrumentHandle, rdBuf, 78, &retCnt)) < 0)
        		return avq8384_status; 

   			if (Scan (rdBuf, "%s>%f,%f,%f,%f,%f,%f",
   					&XData[i],
   					&XJugde[i], 
   					&level[i],
   					&LJudge[i],
   					&SNR[i],
   					&SJudge[i]) < 6)
    			return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		}
	}
	else
	{
  		for (i = 0 ; i < *peakCount; i++)
  		{
  			if ((avq8384_status = viRead (instrumentHandle, rdBuf, 38, &retCnt)) < 0)
        		return avq8384_status; 

   			if (Scan (rdBuf, "%s>%f,%f,%f",&XData[i], &level[i], &SNR[i] ) < 3)
    			return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		}		
	}

	if (head != 0)
	{
		if ((avq8384_status = viWrite (instrumentHandle, "HED1", 4, &retCnt)) < 0)        
        return avq8384_status;	
	}

    return avq8384_status;    
}

/*===========================================================================*/
/* Function: WDM Monitor Number Data			            				*/
/* Purpose:  This function returns the point of WDM Monitor 				*/
/*===========================================================================*/
ViStatus CVIFUNC avq8384_WdmMonNum_Data (ViSession instrumentHandle,
                                         ViInt16 *number)
{
	ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

    if ((avq8384_status = viWrite (instrumentHandle, "OLN?", 4, &retCnt)) < 0)        
        return avq8384_status;                                                    
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status; 

   	if (Scan (rdBuf, "%s>OLN%d[b2]", number) < 1)
   		if (Scan (rdBuf, "%s>%d[b2]", number) < 1)
    		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;    
    
    return avq8384_status;    
}

/*===========================================================================*/
/* Function: WDM Monitor Limit Line Pass Fail Data     						*/
/* Purpose:  This function returns the result of judgement in WDM Monitor. 	*/
/*===========================================================================*/
ViStatus CVIFUNC avq8384_WdmMonLimPf_Data (ViSession instrumentHandle,
                                           ViInt16 *result)
{
	ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

    if ((avq8384_status = viWrite (instrumentHandle, "LRS?", 4, &retCnt)) < 0)        
        return avq8384_status;                                                    
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status; 

   	if (Scan (rdBuf, "%s>%d[b2]",result) < 1)
   		 if (Scan (rdBuf, "%s>LRS%d[b2]",result) < 1)   
    		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;    
    
    return avq8384_status;        
}

/*===========================================================================*/
/* Function: Limit Line Pass Fail Data     									*/
/* Purpose:  This function returns the result of judgement. 				*/
/*===========================================================================*/
ViStatus CVIFUNC avq8384_LimLinePf_Data (ViSession instrumentHandle,
                                         ViInt16 *result)
{
	ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

    if ((avq8384_status = viWrite (instrumentHandle, "LPF?", 4, &retCnt)) < 0)        
        return avq8384_status;                                                    
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status;
        
	if (Scan (rdBuf, "%s>LPF%d[b2]",result) < 1) 
   		if (Scan (rdBuf, "%s>%d[b2]",result) < 1)
    		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;    
    
    return avq8384_status;            
}

/*===========================================================================*/
/* Function: ASE Fit Data Start Point Data            						 */
/* Purpose:  This function returns the data of ASE Fit Data Start Point.	 */
/*===========================================================================*/
ViStatus CVIFUNC avq8384_AseStPoint_Data (ViSession instrumentHandle,
                                          ViInt16 *startPoint)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

    if ((avq8384_status = viWrite (instrumentHandle, "PAS?", 4, &retCnt)) < 0)        
        return avq8384_status;                                                    
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status; 
	
	if (Scan (rdBuf, "%s>PAS%d[b2]", startPoint) < 1)    
   		if (Scan (rdBuf, "%s>%d[b2]", startPoint) < 1)
    		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
  		
    return avq8384_status;
}

/*===========================================================================*/
/* Function: ASE Fit Data Number Data            						*/
/* Purpose:  This function returns the data of ASE Fit Data Number Point.	*/
/*===========================================================================*/

ViStatus CVIFUNC avq8384_AseNum_Data (ViSession instrumentHandle,
                                      ViInt16 *numberDemand)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

    if ((avq8384_status = viWrite (instrumentHandle, "PAN?", 4, &retCnt)) < 0)        
        return avq8384_status;                                                    
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status; 
    
    if (Scan (rdBuf, "%s>PAN%d[b2]", numberDemand) < 1)
   		if (Scan (rdBuf, "%s>%d[b2]", numberDemand) < 1)
    		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
  		
    return avq8384_status;
    
}

/*===========================================================================*/
/* Function: ASE Fit Data Out 						   						*/
/* Purpose:  This function returns the data of ASE Fit Data Out.			*/
/*===========================================================================*/
/*=============== Ascii =================*/
ViStatus CVIFUNC avq8384_AseFit_Data (ViSession instrumentHandle,
                                      ViReal64 aseFitDataOut[])
{
    ViUInt32 retCnt = 0;
    ViInt16 num,i,head;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

	if ((avq8384_status = viWrite (instrumentHandle, "HED?", 4, &retCnt)) < 0)        
        return avq8384_status;
	if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status;
 
   	if (Scan (rdBuf, "%s>HED%d[b2]", &head) < 1)               
   		if (Scan (rdBuf, "%s>%d[b2]", &head) < 1)
			return VI_ERROR_INSTR_INTERPRETING_RESPONSE;	
   
	if ((avq8384_status = viWrite (instrumentHandle, "HED0", 4, &retCnt)) < 0)        
        return avq8384_status;
        
    if ((avq8384_status = avq8384_AseNum_Data (instrumentHandle, &num))<0)
        return avq8384_status; 

	
    /* ASCII Set */ 
    if ((avq8384_status = viWrite (instrumentHandle, "FMT0", 4, &retCnt)) < 0)        
        return avq8384_status;   
        
        
    if ((avq8384_status = viWrite (instrumentHandle, "OPA?", 4, &retCnt)) < 0)        
        return avq8384_status;    
    for (i = 0;i < num;i++)
    {
    	if ((avq8384_status = viRead (instrumentHandle, rdBuf,12, &retCnt)) < 0)
        	return avq8384_status;

   		if (Scan (rdBuf, "%s>%f",&aseFitDataOut[i]) < 1) 
			return VI_ERROR_INSTR_INTERPRETING_RESPONSE;	
     }

	if (head != 0)
	{
		if ((avq8384_status = viWrite (instrumentHandle, "HED1", 4, &retCnt)) < 0)        
        return avq8384_status;	
	}
	

    return avq8384_status;
    
}

/*=============== binary 16bit =================*/ 
ViStatus CVIFUNC avq8384_AseFit16Bit_Data (ViSession instrumentHandle,
                                           short aseFitDataOut[])
{
    ViUInt32 retCnt = 0;
    ViInt16 num,head;
    ViChar rdBuf[BUFFER_SIZE],wrtBuf[100];
    ViStatus avq8384_status = VI_SUCCESS;

	if ((avq8384_status = viWrite (instrumentHandle, "HED?", 4, &retCnt)) < 0)        
        return avq8384_status;
	if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status;
 
   	if (Scan (rdBuf, "%s>HED%d[b2]", &head) < 1)               
   		if (Scan (rdBuf, "%s>%d[b2]", &head) < 1)
			return VI_ERROR_INSTR_INTERPRETING_RESPONSE;	
   
	if ((avq8384_status = viWrite (instrumentHandle, "HED0", 4, &retCnt)) < 0)        
        return avq8384_status;
    
    /* Number? */
    if ((avq8384_status = avq8384_AseNum_Data (instrumentHandle, &num))<0)
        return avq8384_status; 

	
    /* ASCII Set */ 
    if ((avq8384_status = viWrite (instrumentHandle, "FMT1", 4, &retCnt)) < 0)        
        return avq8384_status;   
        
    /* Write */    
    if ((avq8384_status = viWrite (instrumentHandle, "OPA?", 4, &retCnt)) < 0)        
        return avq8384_status;
    
    /* Read */    
	    if ((avq8384_status = viRead (instrumentHandle, rdBuf, (num * 2), &retCnt)) < 0)
        	return avq8384_status;
        	
	Fmt(wrtBuf,"%s%d%s%d%s", "%",num,"d[zb2o10]>%",num,"d[b2]");
	if (Scan (rdBuf, wrtBuf, aseFitDataOut) < 1)
	    	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        

	if (head != 0)
	{
		if ((avq8384_status = viWrite (instrumentHandle, "HED1", 4, &retCnt)) < 0)        
        return avq8384_status;	
	}
	
    return avq8384_status;    
}

/*=============== binary 64bit =================*/  
ViStatus CVIFUNC avq8384_AseFit64bit_Data (ViSession instrumentHandle,
                                           ViReal64 aseFitDataOut[])
{
    ViUInt32 retCnt = 0;
    ViInt16 num,i,head;
 
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

	if ((avq8384_status = viWrite (instrumentHandle, "HED?", 4, &retCnt)) < 0)        
        return avq8384_status;
	if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status;
 
   	if (Scan (rdBuf, "%s>HED%d[b2]", &head) < 1)               
   		if (Scan (rdBuf, "%s>%d[b2]", &head) < 1)
			return VI_ERROR_INSTR_INTERPRETING_RESPONSE;	
   
	if ((avq8384_status = viWrite (instrumentHandle, "HED0", 4, &retCnt)) < 0)        
        return avq8384_status;
    
    /* Number? */
    if ((avq8384_status = avq8384_AseNum_Data (instrumentHandle, &num))<0)
        return avq8384_status; 

	
    /* ASCII Set */ 
    if ((avq8384_status = viWrite (instrumentHandle, "FMT2", 4, &retCnt)) < 0)        
        return avq8384_status;   
        
    /* Write */    
    if ((avq8384_status = viWrite (instrumentHandle, "OPA?", 4, &retCnt)) < 0)        
        return avq8384_status;
    
    /* Read */    
	    if ((avq8384_status = viRead (instrumentHandle, rdBuf, (num * 8), &retCnt)) < 0)
        	return avq8384_status;
    
    /* Set Buffer */
	bin_ViReal64 (num, rdBuf, aseFitDataOut);


	if (head != 0)
	{
		if ((avq8384_status = viWrite (instrumentHandle, "HED1", 4, &retCnt)) < 0)        
        return avq8384_status;	
	}
	
    return avq8384_status;        
}

/*=============== binary 32bit =================*/  
ViStatus CVIFUNC avq8384_AseFit32bit_Data (ViSession instrumentHandle,
                                           float aseFitDataOut[])
{
    ViUInt32 retCnt = 0;
    ViInt16 num,head;
 
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

	if ((avq8384_status = viWrite (instrumentHandle, "HED?", 4, &retCnt)) < 0)        
        return avq8384_status;
	if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status;
 
   	if (Scan (rdBuf, "%s>HED%d[b2]", &head) < 1)               
   		if (Scan (rdBuf, "%s>%d[b2]", &head) < 1)
			return VI_ERROR_INSTR_INTERPRETING_RESPONSE;	
   
	if ((avq8384_status = viWrite (instrumentHandle, "HED0", 4, &retCnt)) < 0)        
        return avq8384_status;
    
    /* Number? */
    if ((avq8384_status = avq8384_AseNum_Data (instrumentHandle, &num))<0)
        return avq8384_status; 

	
    /* ASCII Set */ 
    if ((avq8384_status = viWrite (instrumentHandle, "FMT3", 4, &retCnt)) < 0)        
        return avq8384_status;   
        
    /* Write */    
    if ((avq8384_status = viWrite (instrumentHandle, "OPA?", 4, &retCnt)) < 0)        
        return avq8384_status;
    
    /* Read */    
	if ((avq8384_status = viRead (instrumentHandle, rdBuf, (num * 4), &retCnt)) < 0)
        return avq8384_status;
    
    /* Set Buffer */

	bin_float (num, rdBuf, aseFitDataOut);    

	if (head != 0)
	{
		if ((avq8384_status = viWrite (instrumentHandle, "HED1", 4, &retCnt)) < 0)        
        return avq8384_status;	
	}
	
    return avq8384_status;     
}

/*===========================================================================*/
/* Function: clock Data Out					     							*/
/* Purpose:  This function returns the data of clock.						*/
/*===========================================================================*/

ViStatus CVIFUNC avq8384_DevClock_Data (ViSession instrumentHandle,
                                        ViInt16 *year, ViInt16 *month,
                                        ViInt16 *day, ViInt16 *hour,
                                        ViInt16 *minute, ViInt16 *second)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

    if ((avq8384_status = viWrite (instrumentHandle, "CLO?", 4, &retCnt)) < 0)        
        return avq8384_status;                                                    
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status; 

   	if (Scan (rdBuf, "%s>CLO%d[b2]-%d[b2]-%d[b2],%d[b2]:%d[b2]:%d[b2]",year, month, day, hour, minute, second ) < 6)
   		if (Scan (rdBuf, "%s>%d[b2]-%d[b2]-%d[b2],%d[b2]:%d[b2]:%d[b2]",year, month, day, hour, minute, second) < 6) 
			return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
  		
    return avq8384_status;
    
}
/*===========================================================================*/
/* Function: Instrument ID Data Out					     				*/
/* Purpose:  This function returns the data of Instrument ID.			*/
/*===========================================================================*/

ViStatus CVIFUNC avq8384_InstrID_Data (ViSession instrumentHandle,
                                       ViChar company[], ViChar machineID[],
                                       ViChar serealNo[], ViChar rev1[],
                                       ViChar rev2[])
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE],com[10],mac[6],ser[10],revA[4],revB[4];
    ViStatus avq8384_status = VI_SUCCESS;

    if ((avq8384_status = viWrite (instrumentHandle, "*IDN?", 5, &retCnt)) < 0)        
        return avq8384_status;                                                    
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status; 

   	if (Scan (rdBuf, "%s>%s[w9],%s[w5],%s[w8],%s[w3] %s[w3]",company,machineID ,serealNo,rev1,rev2) < 5)
    			return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
  		
    return avq8384_status;
    
}
/*===========================================================================*/
/* Function: Self Test Data Out					     						*/
/* Purpose:  This function returns the result of Self Test.					*/
/*===========================================================================*/

ViStatus CVIFUNC avq8384_SelfTest_Data (ViSession instrumentHandle,
                                        ViInt16 *selfTestResult)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avq8384_status = VI_SUCCESS;

    if ((avq8384_status = viWrite (instrumentHandle, "*TST?", 5, &retCnt)) < 0)        
        return avq8384_status;                                                    
    if ((avq8384_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avq8384_status; 

   	if (Scan (rdBuf, "%s>%d[b2]",selfTestResult) < 1)
    			return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
  		
    return avq8384_status;
    
}
/*===========================================================================*/
/* Function: Label Edit                                                      */
/* Purpose:  This function set the string of label.                          */
/*===========================================================================*/

ViStatus CVIFUNC avq8384_Label_Set (ViSession instrumentHandle,
                                    ViString labelString)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[53];
    ViStatus avq8384_status = VI_SUCCESS;

    Fmt (wrtBuf, "%s<LAB#%s[w48]#",labelString);    
    total += NumFmtdBytes();
    
    if ((avq8384_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
      	return avq8384_status;

    return avq8384_status;     

    
}

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC avq8384_close (ViSession instrSession)
{
    avq8384_instrRange instrPtr;
    ViSession rmSession;
    ViStatus avq8384_status = VI_SUCCESS;

    if ((avq8384_status = viGetAttribute (instrSession, VI_ATTR_RM_SESSION, &rmSession)) < 0)
        return avq8384_status;
    if ((avq8384_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return avq8384_status;
            
    if (instrPtr != NULL) 
        free (instrPtr);
    
    avq8384_status = viClose (instrSession);
    viClose (rmSession);

    return avq8384_status;
}

/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean avq8384_invalidViBooleanRange (ViBoolean val)
{
    return ((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Signed Integer Value Out Of Range - ViInt16               */
/* Purpose:  This function checks a short signed integer value to see if it  */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avq8384_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avq8384_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Unsigned Char Value Out Of Range - ViUInt8                      */
/* Purpose:  This function checks an unsigned char value to see if it        */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avq8384_invalidViUInt8Range (ViUInt8 val, ViUInt8 min, ViUInt8 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Unsigned Integer Value Out Of Range - ViUInt16            */
/* Purpose:  This function checks a short unsigned integer value to see if it*/  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avq8384_invalidViUInt16Range (ViUInt16 val, ViUInt16 min, ViUInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Unsigned Integer Value Out Of Range - ViUInt32             */
/* Purpose:  This function checks a long unsigned integer value to see if it */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avq8384_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Float) Value Out Of Range - ViReal32                      */
/* Purpose:  This function checks a real (float) value to see if it lies     */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean avq8384_invalidViReal32Range (ViReal32 val, ViReal32 min, ViReal32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean avq8384_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*================================================================================*/
/* Function: Wait On VISA Event                                                   */
/* Purpose:  This function waits for the occurrence of an enabled VISA Event. The */
/*           event type must be enabled before entering this function. Any        */
/*           programmatic commands, actions, or conditions, necessary to generate */
/*           the specified event type, must be satisified before entering this    */
/*           function. The function will wait for the specified timeout and then  */
/*           return. If the specified event is received then the function will    */
/*           return VI_SUCCESS (0), otherwise the status code for the generated   */
/*           error will be returned. In either case the function will disable the */
/*           event type and deallocate the event handle passed from viWaitOnEvent */
/*           before returning. WARNING: If VI_TMO_INFINITE is passed in as the    */
/*           timeout this function WILL NOT return until the specified event is   */
/*           received, if the event is not received the function will not return  */
/*           and it will be necessary to terminate CVI in order to regain control.*/
/*================================================================================*/
ViStatus avq8384_waitOnVisaEvent (ViSession instrSession, ViEventType eventTypeIn, 
            ViUInt32 timeoutIn, ViPUInt16 STB)
{            
    ViStatus avq8384_status = VI_SUCCESS, tempStatus = VI_SUCCESS;
    ViEventType eventTypeOut = 0;
    ViEvent eventHandle = 0;
    
    /* For debug purposes we want to be able to see the status returned by        */
    /* viDisableEvent and viClose if one of the internal functions fails but do   */
    /* not want to return that value as that is not where the initial error       */
    /* occurs in the function, so we assign it to tempStatus.                     */
    
    if ((avq8384_status = viWaitOnEvent (instrSession, eventTypeIn, timeoutIn, 
                                        &eventTypeOut, &eventHandle)) < 0) {
        tempStatus = viDisableEvent (instrSession, eventTypeIn, VI_QUEUE);
        return avq8384_status;
    } 
    
    if (eventTypeIn == VI_EVENT_SERVICE_REQ)
        if ((avq8384_status = viReadSTB (instrSession, STB)) < 0) {
            tempStatus = viClose (eventHandle);
            tempStatus = viDisableEvent (instrSession, eventTypeIn, VI_QUEUE);
            return avq8384_status;
        }   
        
    if ((avq8384_status = viClose (eventHandle)) < 0) {             
        tempStatus = viDisableEvent (instrSession, eventTypeIn, VI_QUEUE);
        return avq8384_status;                 
    }
    
    if ((avq8384_status = viDisableEvent (instrSession, eventTypeIn, VI_QUEUE)) < 0)
        return avq8384_status;                                                   

    return avq8384_status;                                    
}    

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the avq8384_init function.  When   */
/*           an error is detected this function is called to close the       */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from avq8384_init to       */
/*           VI_NULL.                                                        */
/*===========================================================================*/
ViStatus avq8384_initCleanUp (ViSession openRMSession,
                ViPSession openInstrSession, ViStatus currentStatus)
{
    avq8384_instrRange instrPtr;
    
    if (viGetAttribute (*openInstrSession, VI_ATTR_USER_DATA, &instrPtr) >= 0)
        if (instrPtr != NULL) 
            free (instrPtr);

    viClose (*openInstrSession);
    viClose (openRMSession);
    *openInstrSession = VI_NULL;
    
    return currentStatus;
}

/*****************************************************************************/
/*----------- INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINES HERE -------------*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Default Instrument Setup                                        */
/* Purpose:  This function sends a default setup to the instrument.  This    */
/*           function is called by the avq8384_reset operation and by the    */
/*           avq8384_init function if the reset option has not been          */
/*           selected.  This function is useful for configuring any          */
/*           instrument settings that are required by the rest of the        */
/*           instrument driver functions such as turning headers ON or OFF   */
/*           or using the long or short form for commands, queries, and data.*/                                    
/*===========================================================================*/
ViStatus avq8384_defaultInstrSetup (ViSession instrSession)
{
    ViStatus avq8384_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    avq8384_instrRange instrPtr;
    
    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */
    if (avq8384_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr))
        return avq8384_status;
    
    if (instrPtr == NULL) 
        instrPtr = malloc (sizeof (struct avq8384_statusDataRanges));

    instrPtr -> traceRange = 1001;
    
    Fmt (instrPtr -> instrDriverRevision, "%s<%s", avq8384_REVISION);
    
    if ((avq8384_status = viSetAttribute (instrSession, VI_ATTR_USER_DATA, 
                            (ViUInt32)instrPtr)) < 0)
        return avq8384_status;                                       

    return avq8384_status;
}

/*****************************************************************************/
/*=== END INSTRUMENT DRIVER SOURCE CODE =====================================*/
/*****************************************************************************/

